# -*- coding: utf-8 -*-

import system_tests
import unittest

# test needs system_tests.BT.vv['enable_bmff']=1
bSkip=system_tests.BT.verbose_version().get('enable_bmff')!='1'
if bSkip:
    raise unittest.SkipTest('*** requires enable_bmff=1 ***')

class pr_1475_Sony_hif(metaclass=system_tests.CaseMeta):
    url = "https://github.com/Exiv2/exiv2/pull/1475"
    filename = "$data_path/Sony.HIF"
    if bSkip:
        commands=[]
        retval=[]
        stdin=[]
        stderr=[]
        stdout=[]
        print("*** test skipped.  requires enable_bmff=1***")
    else:
        commands = ["$exiv2  -g Image.Make -g Date -g Xm -g Expo -g Flash $filename"
                   ,"$exiv2 -pS $filename"
                   ,"$exiv2 -pX $filename"
                   ,"$exiv2 -pC $filename"
                   ]
        retval = [ 0  ] * len(commands)
        stderr = [ "" ] * len(commands)
        stdin  = [ "" ] * len(commands)
        stdout = ["""Exif.Image.Make                              Ascii       5  SONY
Exif.Image.DateTime                          Ascii      20  2021:02:18 19:55:41
Exif.Photo.ExposureTime                      Rational    1  1/1000 s
Exif.Photo.ExposureProgram                   Short       1  Manual
Exif.Photo.RecommendedExposureIndex          Long        1  100
Exif.Photo.DateTimeOriginal                  Ascii      20  2021:02:18 19:55:41
Exif.Photo.DateTimeDigitized                 Ascii      20  2021:02:18 19:55:41
Exif.Photo.ExposureBiasValue                 SRational   1  0 EV
Exif.Photo.Flash                             Short       1  No, compulsory
Exif.Sony1.FlashExposureComp                 SRational   1  0 EV
Exif.Sony1.ExposureMode                      Short       1  Manual
Exif.Sony1.FlashLevel                        SShort      1  Normal
Exif.Photo.FlashpixVersion                   Undefined   4  1.00
Exif.Photo.ExposureMode                      Short       1  Manual
Xmp.xmp.Rating                               XmpText     1  0
""","""Exiv2::BmffImage::boxHandler: ftyp        0->40 brand: heix
Exiv2::BmffImage::boxHandler: meta       40->2081 
  Exiv2::BmffImage::boxHandler: hdlr       52->33 
  Exiv2::BmffImage::boxHandler: pitm       85->14 
  Exiv2::BmffImage::boxHandler: iinf       99->350 
    Exiv2::BmffImage::boxHandler: infe      113->21 ID =   1 hvc1 
    Exiv2::BmffImage::boxHandler: infe      134->21 ID =   2 hvc1 
    Exiv2::BmffImage::boxHandler: infe      155->21 ID =   3 hvc1 
    Exiv2::BmffImage::boxHandler: infe      176->21 ID =   4 hvc1 
    Exiv2::BmffImage::boxHandler: infe      197->21 ID =   5 hvc1 
    Exiv2::BmffImage::boxHandler: infe      218->21 ID =   6 hvc1 
    Exiv2::BmffImage::boxHandler: infe      239->21 ID =   7 hvc1 
    Exiv2::BmffImage::boxHandler: infe      260->21 ID =   8 hvc1 
    Exiv2::BmffImage::boxHandler: infe      281->21 ID =   9 hvc1 
    Exiv2::BmffImage::boxHandler: infe      302->21 ID =  10 grid 
    Exiv2::BmffImage::boxHandler: infe      323->21 ID =  11 hvc1 
    Exiv2::BmffImage::boxHandler: infe      344->21 ID =  12 hvc1 
    Exiv2::BmffImage::boxHandler: infe      365->21 ID =  13 jpeg 
    Exiv2::BmffImage::boxHandler: infe      386->21 ID =  14 Exif  *** Exif ***
    Exiv2::BmffImage::boxHandler: infe      407->42 ID =  15 mime  *** XMP ***
  Exiv2::BmffImage::boxHandler: iref      449->112 
  Exiv2::BmffImage::boxHandler: iprp      561->1288 
    Exiv2::BmffImage::boxHandler: ipco      569->1184 
      Exiv2::BmffImage::boxHandler: irot      577->9 
      Exiv2::BmffImage::boxHandler: colr      586->19 
      Exiv2::BmffImage::boxHandler: pixi      605->16 
      Exiv2::BmffImage::boxHandler: hvcC      621->345 
      Exiv2::BmffImage::boxHandler: ispe      966->20 pixelWidth_, pixelHeight_ = 2880, 1920
      Exiv2::BmffImage::boxHandler: ispe      986->20 pixelWidth_, pixelHeight_ = 8640, 5760
      Exiv2::BmffImage::boxHandler: hvcC     1006->344 
      Exiv2::BmffImage::boxHandler: ispe     1350->20 pixelWidth_, pixelHeight_ = 1616, 1080
      Exiv2::BmffImage::boxHandler: hvcC     1370->343 
      Exiv2::BmffImage::boxHandler: ispe     1713->20 pixelWidth_, pixelHeight_ = 320, 212
      Exiv2::BmffImage::boxHandler: ispe     1733->20 pixelWidth_, pixelHeight_ = 160, 120
    Exiv2::BmffImage::boxHandler: ipma     1753->96 
  Exiv2::BmffImage::boxHandler: idat     1849->16 
  Exiv2::BmffImage::boxHandler: iloc     1865->256 
      1873 |       16 |   ID |   15 |   4096, 57344
      1889 |       16 |   ID |   14 |  61440, 40960
      1905 |       16 |   ID |   13 | 102400,  4096
      1921 |       16 |   ID |   12 | 106496,  4096
      1937 |       16 |   ID |    1 | 110592,  4562
      1953 |       16 |   ID |    2 | 115154,  2802
      1969 |       16 |   ID |    3 | 117956,  4344
      1985 |       16 |   ID |    4 | 122300,  3772
      2001 |       16 |   ID |    5 | 126072,  2227
      2017 |       16 |   ID |    6 | 128299,  3668
      2033 |       16 |   ID |    7 | 131967,  4518
      2049 |       16 |   ID |    8 | 136485,  2840
      2065 |       16 |   ID |    9 | 139325,  8131
      2081 |       16 |   ID |   10 |      0,     8
      2097 |       16 |   ID |   11 | 147456,  4096
Exiv2::BMFF Exif: ID = 14 from,length = 61440,40960
Exiv2::BMFF XMP: ID = 15 from,length = 4096,57344
Exiv2::BmffImage::boxHandler: free     2121->1967 
Exiv2::BmffImage::boxHandler: mdat     4088->147464 
""","""<?xpacket begin="﻿" id="W5M0MpCehiHzreSzNTczkc9d"?>
<x:xmpmeta xmlns:x="adobe:ns:meta/" x:xmptk="XMP Core 4.4.0-Exiv2">
 <rdf:RDF xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#">
  <rdf:Description rdf:about=""
    xmlns:xmp="http://ns.adobe.com/xap/1.0/"
   xmp:Rating="0"/>
 </rdf:RDF>
</x:xmpmeta>
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                           
<?xpacket end="w"?>""",""]

class pr_1475_Canon_hif(metaclass=system_tests.CaseMeta):
    url = "https://github.com/Exiv2/exiv2/pull/1475"
    filename = "$data_path/Canon.HIF"

    if bSkip:
        commands=[]
        retval=[]
        stdin=[]
        stderr=[]
        stdin=[]
        print("*** test skipped.  requires enable_bmff=1***")
    else:
        commands = ["$exiv2  -g Image.Make -g Date -g Xm -g Expo -g Flash $filename"
                   ,"$exiv2 -pS $filename"
                   ,"$exiv2 -pX $filename"
                   ,"$exiv2 -pC $filename"
                   ]
        retval = [ 0  ] * len(commands)
        stderr = [ "" ] * len(commands)
        stdin  = [ "" ] * len(commands)
        stdout = ["""Exif.Image.Make                              Ascii       6  Canon
Exif.Image.DateTime                          Ascii      20  2021:02:18 19:54:47
Exif.Photo.ExposureTime                      Rational    1  1/1000 s
Exif.Photo.ExposureProgram                   Short       1  Manual
Exif.Photo.RecommendedExposureIndex          Long        1  100
Exif.Photo.DateTimeOriginal                  Ascii      20  2021:02:18 19:54:47
Exif.Photo.DateTimeDigitized                 Ascii      20  2021:02:18 19:54:47
Exif.Photo.ExposureBiasValue                 SRational   1  0 EV
Exif.Photo.Flash                             Short       1  No flash
Exif.CanonCs.FlashMode                       Short       1  Off
Exif.CanonCs.ExposureProgram                 Short       1  Manual (M)
Exif.CanonCs.FlashActivity                   Short       1  Did not fire
Exif.CanonCs.FlashDetails                    Short       1  E-TTL
Exif.CanonCs.ManualFlashOutput               Short       1  n/a
Exif.CanonSi.FlashGuideNumber                Short       1  0
Exif.CanonSi.FlashBias                       Short       1  0 EV
Exif.CanonSi.AutoExposureBracketing          Short       1  Off
Exif.CanonFi.FlashExposureLock               SShort      1  Off
Exif.Photo.FlashpixVersion                   Undefined   4  1.00
Exif.Photo.ExposureMode                      Short       1  Manual
Xmp.xmp.Rating                               XmpText     1  0
""","""Exiv2::BmffImage::boxHandler: ftyp        0->32 brand: heix
Exiv2::BmffImage::boxHandler: meta       32->1163 
  Exiv2::BmffImage::boxHandler: hdlr       44->33 
  Exiv2::BmffImage::boxHandler: uuid       77->62  uuidName cano
    Exiv2::BmffImage::boxHandler: CNCV      101->38 
  Exiv2::BmffImage::boxHandler: dinf      139->36 
  Exiv2::BmffImage::boxHandler: pitm      175->14 
  Exiv2::BmffImage::boxHandler: iinf      189->203 
    Exiv2::BmffImage::boxHandler: infe      203->21 ID =   1 grid 
    Exiv2::BmffImage::boxHandler: infe      224->21 ID = 256 hvc1 
    Exiv2::BmffImage::boxHandler: infe      245->21 ID = 257 hvc1 
    Exiv2::BmffImage::boxHandler: infe      266->21 ID = 258 hvc1 
    Exiv2::BmffImage::boxHandler: infe      287->21 ID = 259 hvc1 
    Exiv2::BmffImage::boxHandler: infe      308->21 ID = 512 hvc1 
    Exiv2::BmffImage::boxHandler: infe      329->21 ID = 768 Exif  *** Exif ***
    Exiv2::BmffImage::boxHandler: infe      350->42 ID = 769 mime  *** XMP ***
  Exiv2::BmffImage::boxHandler: iref      392->74 
  Exiv2::BmffImage::boxHandler: iprp      466->569 
    Exiv2::BmffImage::boxHandler: ipco      474->507 
      Exiv2::BmffImage::boxHandler: hvcC      482->176 
      Exiv2::BmffImage::boxHandler: ispe      658->20 pixelWidth_, pixelHeight_ = 1216, 832
      Exiv2::BmffImage::boxHandler: colr      678->19 
      Exiv2::BmffImage::boxHandler: pixi      697->16 
      Exiv2::BmffImage::boxHandler: ispe      713->20 pixelWidth_, pixelHeight_ = 2400, 1600
      Exiv2::BmffImage::boxHandler: irot      733->9 
      Exiv2::BmffImage::boxHandler: hvcC      742->175 
      Exiv2::BmffImage::boxHandler: ispe      917->20 pixelWidth_, pixelHeight_ = 320, 214
      Exiv2::BmffImage::boxHandler: colr      937->19 
      Exiv2::BmffImage::boxHandler: pixi      956->16 
      Exiv2::BmffImage::boxHandler: irot      972->9 
    Exiv2::BmffImage::boxHandler: ipma      981->54 
  Exiv2::BmffImage::boxHandler: idat     1035->16 
  Exiv2::BmffImage::boxHandler: iloc     1051->144 
      1059 |       16 |   ID |    1 |      0,     8
      1075 |       16 |   ID |  256 |  46080,344284
      1091 |       16 |   ID |  257 | 390364,340989
      1107 |       16 |   ID |  258 | 731353,257177
      1123 |       16 |   ID |  259 | 988530,264862
      1139 |       16 |   ID |  512 |  32256, 10284
      1155 |       16 |   ID |  768 |   1536, 30463
      1171 |       16 |   ID |  769 |  43008,  3072
Exiv2::BMFF Exif: ID = 768 from,length = 1536,30463
Exiv2::BMFF XMP: ID = 769 from,length = 43008,3072
Exiv2::BmffImage::boxHandler: mdat     1195->1252197 
""","""<?xpacket begin="﻿" id="W5M0MpCehiHzreSzNTczkc9d"?>
<x:xmpmeta xmlns:x="adobe:ns:meta/" x:xmptk="XMP Core 4.4.0-Exiv2">
 <rdf:RDF xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#">
  <rdf:Description rdf:about=""
    xmlns:xmp="http://ns.adobe.com/xap/1.0/"
   xmp:Rating="0"/>
 </rdf:RDF>
</x:xmpmeta>
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                           
<?xpacket end="w"?>""",""]




# -*- coding: utf-8 -*-

import system_tests


class TestCvePoC(metaclass=system_tests.CaseMeta):

    url = "https://github.com/Exiv2/exiv2/issues/202"
    cve_url = "http://cve.mitre.org/cgi-bin/cvename.cgi?name=CVE-2018-4868"
    found_by = ["afl", "topsecLab", "xcainiao"]

    filename = "$data_path/exiv2-memorymmap-error"
    commands = ["$exiv2 " + filename]
    stdout = [""]
    stderr = ["""$exiv2_exception_message """ + filename + """:
$kerCorruptedMetadata
"""]
    retval = [1]



# -*- coding: utf-8 -*-

from system_tests import CaseMeta, CopyTmpFiles, path
@CopyTmpFiles("$data_path/issue_ghsa_9jh3_fcc3_g6hv_poc.jpg")

class JpegBasePrintStructureInfiniteLoop(metaclass=CaseMeta):
    """
    Regression test for the bug described in:
    https://github.com/Exiv2/exiv2/security/advisories/GHSA-9jh3-fcc3-g6hv
    """
    url = "https://github.com/Exiv2/exiv2/security/advisories/GHSA-9jh3-fcc3-g6hv"

    filename = path("$tmp_path/issue_ghsa_9jh3_fcc3_g6hv_poc.jpg")
    commands = ["$exiv2 -d I rm $filename"]
    stdout = [""]
    stderr = [""]
    retval = [0]



# -*- coding: utf-8 -*-

from system_tests import CaseMeta, path


class CanonEOSM100(metaclass=CaseMeta):

    filename = path("$data_path/exiv2-pr317.exv")
    commands = ["$exiv2 -pa --grep model/i $filename"]

    stdout = ["""Exif.Image.Model                             Ascii      15  Canon EOS M100
Exif.Canon.ModelID                           Long        1  EOS M100
Exif.Photo.LensModel                         Ascii      29  EF-M15-45mm f/3.5-6.3 IS STM
"""
    ]
    stderr = [""]
    retval = [0]



# -*- coding: utf-8 -*-

import system_tests


class CheckShutterInPentax(metaclass=system_tests.CaseMeta):

    url = "http://dev.exiv2.org/issues/1242"
    filename = "$data_path/Reagan.jp2"

    commands = ["$exiv2 -pa -b $filename"]

    stdout = [r"""Exif.Image.ImageDescription                  Ascii     403  040621-N-6536T-062
USS Ronald Reagan (CVN 76), June 21, 2004 -  USS Ronald Reagan (CVN 76) sails through the Straits of Magellan on its way to the Pacific Ocean. The NavyÃ­s newest aircraft carrier is underway circumnavigating South America in transit to its new homeport of San Diego. U.S. Navy photo by PhotographerÃ­s Mate 3rd Class (AW) Elizabeth Thompson. (RELEASE)
                               
Exif.Image.Make                              Ascii      18  NIKON CORPORATION
Exif.Image.Model                             Ascii      10  NIKON D1X
Exif.Image.Orientation                       Short       1  top, left
Exif.Image.XResolution                       Rational    1  300
Exif.Image.YResolution                       Rational    1  300
Exif.Image.ResolutionUnit                    Short       1  inch
Exif.Image.Software                          Ascii      40  Adobe Photoshop Elements 12.0 Macintosh
Exif.Image.DateTime                          Ascii      20  2016:09:13 11:58:16
Exif.Image.Artist                            Ascii      34  PhotographerÃ­s Mate 3rd Class (A
Exif.Image.YCbCrPositioning                  Short       1  Centered
Exif.Image.ExifTag                           Long        1  712
Exif.Photo.ExposureTime                      Rational    1  1/125 s
Exif.Photo.FNumber                           Rational    1  F5
Exif.Photo.ExposureProgram                   Short       1  Manual
Exif.Photo.ExifVersion                       Undefined   4  2.20
Exif.Photo.DateTimeOriginal                  Ascii      20  2004:06:21 23:37:53
Exif.Photo.DateTimeDigitized                 Ascii      20  2004:06:21 23:37:53
Exif.Photo.ComponentsConfiguration           Undefined   4  YCbCr
Exif.Photo.ShutterSpeedValue                 SRational   1  1/125 s
Exif.Photo.ApertureValue                     Rational    1  F5
Exif.Photo.ExposureBiasValue                 SRational   1  +1/3 EV
Exif.Photo.MaxApertureValue                  Rational    1  F2.8
Exif.Photo.MeteringMode                      Short       1  Center weighted average
Exif.Photo.LightSource                       Short       1  Cloudy weather
Exif.Photo.Flash                             Short       1  No flash
Exif.Photo.FocalLength                       Rational    1  42.0 mm
Exif.Photo.SubSecTime                        Ascii       3  06
Exif.Photo.SubSecTimeOriginal                Ascii       3  06
Exif.Photo.SubSecTimeDigitized               Ascii       3  06
Exif.Photo.FlashpixVersion                   Undefined   4  1.00
Exif.Photo.ColorSpace                        Short       1  Uncalibrated
Exif.Photo.PixelXDimension                   Short       1  200
Exif.Photo.PixelYDimension                   Short       1  130
Exif.Photo.SensingMethod                     Short       1  One-chip color area
Exif.Photo.FileSource                        Undefined   1  Digital still camera
Exif.Photo.SceneType                         Undefined   1  Directly photographed
Exif.Photo.CustomRendered                    Short       1  Normal process
Exif.Photo.ExposureMode                      Short       1  Manual
Exif.Photo.WhiteBalance                      Short       1  Manual
Exif.Photo.DigitalZoomRatio                  Rational    1  1.0
Exif.Photo.FocalLengthIn35mmFilm             Short       1  63.0 mm
Exif.Photo.SceneCaptureType                  Short       1  Standard
Exif.Photo.GainControl                       Short       1  None
Exif.Photo.Contrast                          Short       1  Normal
Exif.Photo.Saturation                        Short       1  Normal
Exif.Photo.Sharpness                         Short       1  Normal
Exif.Photo.SubjectDistanceRange              Short       1  Unknown
Exif.Photo.ImageUniqueID                     Ascii      33  127c1377b054a3f65bf2754ebb24e7f2
Exif.Image.GPSTag                            Long        1  1300
Exif.GPSInfo.GPSVersionID                    Byte        4  2.2.0.0
Iptc.Application2.RecordVersion              Short       1  2
Iptc.Application2.ObjectName                 String     18  040621-N-6536T-062
Iptc.Application2.Urgency                    String      1  5
Iptc.Application2.Category                   String      1  N
Iptc.Application2.SuppCategory               String     12  703-614-9154
Iptc.Application2.SuppCategory               String     23  navyvisualnews@navy.mil
Iptc.Application2.SuppCategory               String     11  UNCLASSFIED
Iptc.Application2.Keywords                   String     13  ronald reagan
Iptc.Application2.Keywords                   String      6  reagan
Iptc.Application2.Keywords                   String      6  cvn 76
Iptc.Application2.Keywords                   String      6  cvn-76
Iptc.Application2.Keywords                   String     18  straights magellan
Iptc.Application2.Keywords                   String      8  magellan
Iptc.Application2.Keywords                   String      7  carrier
Iptc.Application2.Keywords                   String     12  nimitz-class
Iptc.Application2.Keywords                   String      4  ship
Iptc.Application2.Keywords                   String      8  underway
Iptc.Application2.SpecialInstructions        String     49  Credit as U.S. Navy photo by Elizabeth Thompson. 
Iptc.Application2.DateCreated                Date        8  2004-06-21
Iptc.Application2.TimeCreated                Time       11  00:00:00+00:00
Iptc.Application2.DigitizationDate           Date        8  2004-06-21
Iptc.Application2.DigitizationTime           Time       11  23:37:53-04:00
Iptc.Application2.Byline                     String     31  Photographerís Mate 3rd Class (
Iptc.Application2.BylineTitle                String     21  U.S Navy Photographer
Iptc.Application2.City                       String     19  Straits of Magellan
Iptc.Application2.CountryName                String     13  South America
Iptc.Application2.Credit                     String      8  U.S Navy
Iptc.Application2.Source                     String     24  Navy Visual News Service
Iptc.Application2.Caption                    String    400  040621-N-6536T-062
USS Ronald Reagan (CVN 76), June 21, 2004 -  USS Ronald Reagan (CVN 76) sails through the Straits of Magellan on its way to the Pacific Ocean. The Navyís newest aircraft carrier is underway circumnavigating South America in transit to its new homeport of San Diego. U.S. Navy photo by Photographerís Mate 3rd Class (AW) Elizabeth Thompson. (RELEASE)
                               
Iptc.Application2.Writer                     String      9  Dir. NVNS
Xmp.xmp.ModifyDate                           XmpText    25  2016-09-13T11:58:16+01:00
Xmp.xmp.CreateDate                           XmpText    25  2004-06-21T23:37:53+01:00
Xmp.xmp.MetadataDate                         XmpText    25  2016-09-13T11:58:16+01:00
Xmp.xmp.CreatorTool                          XmpText    38  Adobe Photoshop Elements 6.0 Macintosh
Xmp.photoshop.Instructions                   XmpText    49  Credit as U.S. Navy photo by Elizabeth Thompson. 
Xmp.photoshop.CaptionWriter                  XmpText     9  Dir. NVNS
Xmp.photoshop.Urgency                        XmpText     1  5
Xmp.photoshop.City                           XmpText    19  Straits of Magellan
Xmp.photoshop.Category                       XmpText     1  N
Xmp.photoshop.Country                        XmpText    13  South America
Xmp.photoshop.Credit                         XmpText     8  U.S Navy
Xmp.photoshop.AuthorsPosition                XmpText    21  U.S Navy Photographer
Xmp.photoshop.DateCreated                    XmpText    10  2004-06-21
Xmp.photoshop.Source                         XmpText    24  Navy Visual News Service
Xmp.photoshop.LegacyIPTCDigest               XmpText    32  977177A6C759A2BBD07317E3D5921073
Xmp.photoshop.ColorMode                      XmpText     1  3
Xmp.photoshop.ICCProfile                     XmpText    16  Adobe RGB (1998)
Xmp.photoshop.SupplementalCategories         XmpBag      3  703-614-9154, navyvisualnews@navy.mil, UNCLASSFIED
Xmp.xmpMM.InstanceID                         XmpText    40  xmp.iid:F77F117407206811822A8C00775B3FDC
Xmp.xmpMM.DocumentID                         XmpText    37  uuid:D6CBDC1D8DF2E511BA6BFBE914561F6D
Xmp.xmpMM.OriginalDocumentID                 XmpText    37  uuid:D6CBDC1D8DF2E511BA6BFBE914561F6D
Xmp.xmpMM.DerivedFrom                        XmpText     0  type="Struct"
Xmp.xmpMM.DerivedFrom/stRef:instanceID       XmpText    41  uuid:ec11a6b0-cc13-11d8-9c21-fa22e28297f6
Xmp.xmpMM.DerivedFrom/stRef:documentID       XmpText    58  adobe:docid:photoshop:1c90e091-c489-11d8-ad7d-b4c1b2598b09
Xmp.xmpMM.History                            XmpText     0  type="Seq"
Xmp.xmpMM.History[1]                         XmpText     0  type="Struct"
Xmp.xmpMM.History[1]/stEvt:action            XmpText     5  saved
Xmp.xmpMM.History[1]/stEvt:instanceID        XmpText    40  xmp.iid:F77F117407206811822A8C00775B3FDC
Xmp.xmpMM.History[1]/stEvt:when              XmpText    25  2016-09-13T11:58:16+01:00
Xmp.xmpMM.History[1]/stEvt:softwareAgent     XmpText    39  Adobe Photoshop Elements 12.0 Macintosh
Xmp.xmpMM.History[1]/stEvt:changed           XmpText     1  /
Xmp.dc.format                                XmpText    10  image/jpeg
Xmp.dc.description                           LangAlt     1  lang="x-default" 040621-N-6536T-062
USS Ronald Reagan (CVN 76), June 21, 2004 -  USS Ronald Reagan (CVN 76) sails through the Straits of Magellan on its way to the Pacific Ocean. The NavyÃ­s newest aircraft carrier is underway circumnavigating South America in transit to its new homeport of San Diego. U.S. Navy photo by PhotographerÃ­s Mate 3rd Class (AW) Elizabeth Thompson. (RELEASE)
                               
Xmp.dc.title                                 LangAlt     1  lang="x-default" 040621-N-6536T-062
Xmp.dc.creator                               XmpSeq      1  PhotographerÃ­s Mate 3rd Class (A
Xmp.dc.subject                               XmpBag     10  ronald reagan, reagan, cvn 76, cvn-76, straights magellan, magellan, carrier, nimitz-class, ship, underway
Xmp.xmpRights.Marked                         XmpText     5  False
"""
    ]

    def compare_stdout(self, i, command, got_stdout, expected_stdout):
        """
        The metadata in this file contain badly encoded strings with linefeeds
        only resulting in un-comparable strings. Here we check line by line
        instead, which removes the nasty line ending problems.
        """
        for i, (got_line, expected_line) in \
            enumerate(zip(got_stdout.splitlines(), expected_stdout.splitlines())):
            self.assertEqual(
                got_line, expected_line, msg="error in line {:d}".format(i)
            )

    stderr = [""]
    retval = [0]



# -*- coding: utf-8 -*-

import system_tests
import unittest

@unittest.skip("Skipping test using option -pR (only for Debug mode)")
class ThrowsWhenIFDsAreMalformed(metaclass=system_tests.CaseMeta):
    url = "https://github.com/Exiv2/exiv2/issues/511"

    filename = system_tests.path("$data_path/pocIssue511")
    commands = ["$exiv2 -pR " + filename]
    retval = [1]
    stdout = ["""STRUCTURE OF TIFF FILE (II): """ + filename + """
"""]
    stderr = ["""$exiv2_exception_message """ + filename + """:
$kerCorruptedMetadata
"""]



# -*- coding: utf-8 -*-

from system_tests import CaseMeta, CopyTmpFiles, path
@CopyTmpFiles("$data_path/test_issue_1471.exv")

class test_issue_1471Test(metaclass=CaseMeta):

    filename  = path("$tmp_path/test_issue_1471.exv")
    commands  = [  "$exiv2 -K Exif.Sony2010e.WB_RGBLevels              $filename"
                ,  "$exiv2 -M\"set Exif.Image.ImageID foobar\"         $filename"
                ,  "$exiv2 -K Exif.Sony2010e.WB_RGBLevels              $filename"
                ]
    stdout   =  ["Exif.Sony2010e.WB_RGBLevels                  Short       3  598 256 442\n"
                ,""
                ,"Exif.Sony2010e.WB_RGBLevels                  Short       3  598 256 442\n"
                ]
    stderr = [""]*len(commands)
    retval = [ 0]*len(commands)



# -*- coding: utf-8 -*-

from system_tests import CaseMeta, CopyTmpFiles, path
@CopyTmpFiles("$data_path/issue_1881_poc.jpg", "$data_path/issue_1881_coverage.jpg")

class SonyPreviewImageLargeAllocation(metaclass=CaseMeta):
    """
    Regression test for the bug described in:
    https://github.com/Exiv2/exiv2/issues/1881
    """
    url = "https://github.com/Exiv2/exiv2/issues/1881"

    filename1 = path("$tmp_path/issue_1881_poc.jpg")
    filename2 = path("$tmp_path/issue_1881_coverage.jpg")
    commands = ["$exiv2 -q -d I rm $filename1", "$exiv2 -q -d I rm $filename2"]
    stdout = ["",""]
    stderr = [
"""Exiv2 exception in erase action for file $filename1:
$kerCorruptedMetadata
""",
""]
    retval = [1,0]



# -*- coding: utf-8 -*-

from system_tests import CaseMeta, path


class OutOfMemoryInRafImageReadMetadata(metaclass=CaseMeta):
    """
    Regression test for the bug described in:
    https://github.com/Exiv2/exiv2/issues/857

    There is no bounds check on the value of jpg_img_len in
    RafImage::readMetadata(), leading to an out-of-memory error.
    """
    url = "https://github.com/Exiv2/exiv2/issues/857"

    filename1 = path("$data_path/issue_857_poc.raf")
    filename2 = path("$data_path/issue_857_coverage.raf")
    commands = ["$exiv2 $filename1", "$exiv2 $filename2"]
    stdout = ["", ""]
    stderr = [
"""Exiv2 exception in print action for file $filename1:
$kerCorruptedMetadata
""",
"""Exiv2 exception in print action for file $filename2:
This does not look like a TIFF image
"""
]
    retval = [1,1]



import os
import re
import unittest

from system_tests import BT


class TestCases(unittest.TestCase):


    def setUp(self):
        BT.Config.init()


    def tearDown(self):
        pass


    def addmoddel_test(self):
        # Test driver to run the addmoddel sample program
        jpg      = 'exiv2-empty.jpg'
        BT.copyTestFile(jpg)
        out      = BT.Output()
        out     += BT.Executer('addmoddel {jpg}', vars())
        out     += BT.Executer('exiv2 -pv {jpg}', vars())
        BT.reportTest('addmoddel', out)


    def conversions_test(self):
        # XMP parser test driver
        jpg      = 'exiv2-empty.jpg'
        out      = BT.Output()

        BT.log.info('#1 Convert Exif ImageDescription to XMP x-default langAlt value')
        out     += 'Testcase 1'
        out     += '=========='  # 9 equal signs
        BT.copyTestFile(jpg, 'h.jpg')
        out     += BT.Executer("exiv2 -M'set Exif.Image.ImageDescription The Exif image description' h.jpg")
        BT.rm('h.xmp')
        out     += BT.Executer('exiv2 -eX h.jpg')
        out     += BT.Executer('exiv2 -px h.xmp')
        out     += BT.Executer('exiv2 -PEkycv h.xmp')
        out     += BT.Executer('exiv2 -pi h.xmp')

        BT.log.info('#2 Convert XMP x-default langAlt value back to Exif ImageDescription')
        out     += ''
        out     += 'Testcase 2'
        out     += '=========='
        BT.copyTestFile(jpg, 'i.jpg')
        BT.cp('h.xmp', 'i.xmp')
        out     += BT.Executer('exiv2 -iX  i.jpg')
        out     += BT.Executer('exiv2 -px  i.jpg')
        out     += BT.Executer('exiv2 -PEkycv i.jpg')
        out     += BT.Executer('exiv2 -pi  i.jpg')

        BT.log.info('#3 Convert XMP single non-x-default langAlt value to Exif ImageDescription')
        out     += ''
        out     += 'Testcase 3'
        out     += '=========='
        BT.save(BT.cat('i.xmp').replace('x-default', 'de-DE'), 'j.xmp')
        BT.copyTestFile(jpg, 'j.jpg')
        out     += BT.Executer('exiv2 -iX j.jpg')
        out     += BT.Executer('exiv2 -px j.jpg')
        out     += BT.Executer('exiv2 -PEkycv j.jpg')
        out     += BT.Executer('exiv2 -pi j.jpg')

        BT.log.info("#4 This shouldn't work: No x-default, more than one language")
        out     += ''
        out     += 'Testcase 4'
        out     += '=========='
        BT.save(BT.cat('j.xmp').replace('<rdf:li xml:lang="de-DE">The Exif image description</rdf:li>',
                                        '<rdf:li xml:lang="de-DE">The Exif image description</rdf:li><rdf:li xml:lang="it-IT">Ciao bella</rdf:li>')
                ,'k.xmp')
        BT.copyTestFile(jpg, 'k.jpg')
        out     += BT.Executer('exiv2 -iX k.jpg')
        out     += BT.Executer('exiv2 -px k.jpg')
        out     += BT.Executer('exiv2 -v -PEkycv k.jpg')
        out     += BT.Executer('exiv2 -v -pi k.jpg')

        BT.log.info('#5 Add a default language to the XMP file and convert to Exif and IPTC')
        out     += ''
        out     += 'Testcase 5'
        out     += '=========='
        BT.cp('k.xmp', 'l.xmp')
        out     += BT.Executer('''exiv2 -M'set Xmp.dc.description lang="x-default" How to fix this mess' l.xmp''')
        out     += BT.grep('x-default', 'l.xmp')
        BT.copyTestFile(jpg, 'l.jpg')
        out     += BT.Executer('exiv2 -iX l.jpg')
        out     += BT.Executer('exiv2 -px -b l.jpg')
        out     += BT.Executer('exiv2 -PEkycv l.jpg')
        out     += BT.Executer('exiv2 -pi l.jpg')

        BT.log.info('#6 Convert an Exif user comment to XMP')
        out     += ''
        out     += 'Testcase 6'
        out     += '=========='
        BT.copyTestFile(jpg, 'm.jpg')
        out     += BT.Executer("exiv2 -M'set Exif.Photo.UserComment charset=Jis This is a JIS encoded Exif user comment. Or was it?' m.jpg")
        out     += BT.Executer('exiv2 -PEkycv m.jpg')
        BT.rm('m.xmp')
        out     += BT.Executer('exiv2 -eX m.jpg')
        out     += BT.Executer('exiv2 -px m.xmp')
        out     += BT.Executer('exiv2 -PEkycv m.xmp')
        out     += BT.Executer('exiv2 -v -pi m.xmp')

        BT.log.info('#7 And back to Exif')
        out     += ''
        out     += 'Testcase 7'
        out     += '=========='
        BT.copyTestFile(jpg, 'n.jpg')
        BT.cp('m.xmp', 'n.xmp')
        out     += BT.Executer('exiv2 -iX n.jpg')
        out     += BT.Executer('exiv2 -px n.jpg')
        out     += BT.Executer('exiv2 -PEkycv n.jpg')
        out     += BT.Executer('exiv2 -v -pi n.jpg')

        BT.log.info('#8 Convert IPTC keywords to XMP')
        out     += ''
        out     += 'Testcase 8'
        out     += '=========='
        BT.copyTestFile(jpg, 'o.jpg')
        out     += BT.Executer('''exiv2 -M'add Iptc.Application2.Keywords Sex' o.jpg''')
        out     += BT.Executer('''exiv2 -M'add Iptc.Application2.Keywords Drugs' o.jpg''')
        out     += BT.Executer('''exiv2 -M"add Iptc.Application2.Keywords Rock'n'roll" o.jpg''')
        out     += BT.Executer('''exiv2 -pi o.jpg''')
        BT.rm('o.xmp')
        out     += BT.Executer('exiv2 -eX o.jpg')
        out     += BT.Executer('exiv2 -px o.xmp')
        out     += BT.Executer('exiv2 -v -PEkycv o.xmp')
        out     += BT.Executer('exiv2 -pi o.xmp')

        BT.log.info('#9 And back to IPTC')
        out     += ''
        out     += 'Testcase 9'
        out     += '=========='
        BT.copyTestFile(jpg, 'p.jpg')
        BT.cp('o.xmp', 'p.xmp')
        out     += BT.Executer('exiv2 -iX p.jpg')
        out     += BT.Executer('exiv2 -px p.jpg')
        out     += BT.Executer('exiv2 -v -PEkycv p.jpg')
        out     += BT.Executer('exiv2 -pi p.jpg')

        BT.log.info('#10 Convert an Exif tag to an XMP text value')
        out     += ''
        out     += 'Testcase 10'
        out     += '==========='  # 10 equal signs
        BT.copyTestFile(jpg, 'q.jpg')
        out     += BT.Executer("exiv2 -M'set Exif.Image.Software Exiv2' q.jpg")
        out     += BT.Executer("exiv2 -PEkycv q.jpg")
        BT.rm('q.xmp')
        out     += BT.Executer('exiv2 -eX q.jpg')
        out     += BT.Executer('exiv2 -px q.xmp')
        out     += BT.Executer('exiv2 -PEkycv q.xmp')
        out     += BT.Executer('exiv2 -v -pi q.xmp')

        BT.log.info('#11 And back to Exif')
        out     += ''
        out     += 'Testcase 11'
        out     += '==========='
        BT.copyTestFile(jpg, 'r.jpg')
        BT.cp('q.xmp', 'r.xmp')
        out     += BT.Executer('exiv2 -iX r.jpg')
        out     += BT.Executer('exiv2 -px r.jpg')
        out     += BT.Executer('exiv2 -PEkycv r.jpg')
        out     += BT.Executer('exiv2 -v -pi r.jpg')

        BT.log.info('#12 Convert an IPTC dataset to an XMP text value')
        out     += ''
        out     += 'Testcase 12'
        out     += '==========='
        BT.copyTestFile(jpg, 's.jpg')
        out     += BT.Executer("exiv2 -M'set Iptc.Application2.SubLocation Kuala Lumpur' s.jpg")
        out     += BT.Executer("exiv2 -pi s.jpg")
        BT.rm('s.xmp')
        out     += BT.Executer('exiv2 -eX s.jpg')
        out     += BT.Executer('exiv2 -px s.xmp')
        out     += BT.Executer('exiv2 -v -PEkycv s.xmp')
        out     += BT.Executer('exiv2 -pi s.xmp')

        BT.log.info('#13 And back to IPTC')
        out     += ''
        out     += 'Testcase 13'
        out     += '==========='
        BT.copyTestFile(jpg, 't.jpg')
        BT.cp('s.xmp', 't.xmp')
        out     += BT.Executer('exiv2 -iX t.jpg')
        out     += BT.Executer('exiv2 -px t.jpg')
        out     += BT.Executer('exiv2 -v -PEkycv t.jpg')
        out     += BT.Executer('exiv2 -pi t.jpg')

        BT.log.info('#14 Convert a few other tags of interest from Exif/IPTC to XMP')
        out     += ''
        out     += 'Testcase 14'
        out     += '==========='
        BT.copyTestFile(jpg, 'u.jpg')
        out     += BT.Executer("exiv2 -M'set Exif.Photo.DateTimeOriginal 2003:12:14 12:01:44' u.jpg")
        out     += BT.Executer("exiv2 -M'set Exif.Photo.SubSecTimeOriginal 999999999' u.jpg")
        out     += BT.Executer("exiv2 -M'set Exif.Photo.ExifVersion 48 50 50 49' u.jpg")
        out     += BT.Executer("exiv2 -M'set Exif.Photo.ComponentsConfiguration 1 2 3 0' u.jpg")
        out     += BT.Executer("exiv2 -M'set Exif.Photo.Flash 73' u.jpg")
        out     += BT.Executer("exiv2 -M'set Exif.GPSInfo.GPSLatitude 3/1 8/1 29734512/1000000' u.jpg")
        out     += BT.Executer("exiv2 -M'set Exif.GPSInfo.GPSLatitudeRef N' u.jpg")
        out     += BT.Executer("exiv2 -M'set Exif.GPSInfo.GPSVersionID 2 2 0 1' u.jpg")
        out     += BT.Executer("exiv2 -M'set Exif.GPSInfo.GPSTimeStamp 1/1 2/1 999999999/1000000000' u.jpg")
        out     += BT.Executer('exiv2 -PEkycv u.jpg')
        out     += BT.Executer('exiv2 -pi u.jpg')
        BT.rm('u.xmp')
        out     += BT.Executer('exiv2 -eX u.jpg')
        out     += BT.Executer('exiv2 -px u.xmp')
        out     += BT.Executer('exiv2 -PEkycv u.xmp')
        out     += BT.Executer('exiv2 -pi u.xmp')

        BT.log.info('#15 And back to Exif/IPTC')
        out     += ''
        out     += 'Testcase 15'
        out     += '==========='
        BT.copyTestFile(jpg, 'v.jpg')
        BT.cp('u.xmp', 'v.xmp')
        out     += BT.Executer("exiv2 -M'set Xmp.xmp.ModifyDate 2015-04-17T18:10:22Z' v.xmp")
        out     += BT.Executer('exiv2 -iX v.jpg')  # need TZ=GMT-8
        out     += BT.Executer('exiv2 -px v.jpg')
        out     += BT.Executer('exiv2 -PEkycv v.jpg').stdout.replace('17 19:10:22', '18 02:10:22') # evade this test on MSVC builds (Issue #485)
        out     += BT.Executer('exiv2 -pi v.jpg')

        BT.log.info('#16 https://github.com/Exiv2/exiv2/issues/521')
        out     += ''
        out     += 'Testcase 16'
        out     += '==========='
        BT.copyTestFile('DSC_3079.jpg')
        out     += BT.Executer('exiv2 -px                        DSC_3079.jpg')
        out     += BT.Executer('exiv2 -M"del Xmp.mwg-rs.Regions" DSC_3079.jpg')
        out     += BT.Executer('exiv2 -px                        DSC_3079.jpg')

        # Ignore output differences on Windows
        for pair in [
            ('charset="Jis"'    , 'charset=Jis'),
            ('charset="Unicode"', 'charset=Unicode'),
            (' 9  Rocknroll'    , "11  Rock'n'roll"),
            ('Rocknroll'        , "Rock'n'roll")
        ]:
            out  = str(out).replace(pair[0], pair[1])

        BT.reportTest('conversions', out)


    def crw_test(self):
        # Test driver for CRW file operations
        crwfile  = 'exiv2-canon-powershot-s40.crw'

        BT.log.info('#1 Add and modify tags')
        cmds     = '''
set Exif.Photo.ColorSpace 65535
set Exif.Canon.OwnerName Different owner
set Exif.Canon.FirmwareVersion Whatever version
set Exif.Canon.SerialNumber 1
add Exif.Canon.SerialNumber 2
set Exif.Photo.ISOSpeedRatings 155
set Exif.Photo.DateTimeOriginal 2007:11:11 09:10:11
set Exif.Image.DateTime 2020:05:26 07:31:41
set Exif.Photo.DateTimeDigitized 2020:05:26 07:31:42
'''.lstrip('\n')
        cmdfile  = 'cmdfile1'
        BT.save(cmds, cmdfile)
        BT.copyTestFile(crwfile)
        out      = BT.Output()
        out     += BT.Executer('exiv2 -v -pt           {crwfile}', vars())
        out     += BT.Executer('exiv2 -v -m{cmdfile}   {crwfile}', vars())
        out     += BT.Executer('exiv2 -v -pt           {crwfile}', vars())

        BT.log.info('#2 Delete tags')
        BT.copyTestFile(crwfile)
        out     += BT.Executer("exiv2 -v -pt           {crwfile}", vars())
        out     += BT.Executer("exiv2 -v -M'del Exif.Canon.OwnerName'    {crwfile}", vars())
        out     += BT.Executer("exiv2 -v -pt           {crwfile}", vars())

        # sed evades TZ issue on MSVC builds #1221
        out      = str(out).replace('23 19:54', '23 18:54').replace('24 01:54', '23 18:54')

        BT.reportTest('crw-test', out)


    def exifdata_test(self):
        # Test driver for exifdata copy construction and assignment unit tests
        out      = BT.Output()
        for jpg in ['exiv2-gc.jpg', 'exiv2-canon-powershot-s40.jpg', 'exiv2-nikon-d70.jpg']:
            BT.copyTestFile(jpg)
            out += BT.Executer('exifdata-test {jpg}', vars())
        BT.reportTest('exifdata-test', out)


    def exiv2_test(self):
        # Add each image to the following three lists.
        # The image basename in the second and third lists
        # is the Exif timestamp adjusted by -12:01:01.
        images_1 = [
            'exiv2-empty.jpg'
            ,'exiv2-canon-powershot-s40.jpg'
            ,'exiv2-nikon-e990.jpg'
            ,'exiv2-nikon-d70.jpg'
            ,'exiv2-nikon-e950.jpg'
            ,'exiv2-canon-eos-300d.jpg'
            ,'exiv2-kodak-dc210.jpg'
            ,'exiv2-fujifilm-finepix-s2pro.jpg'
            ,'exiv2-sigma-d10.jpg'
            ,'exiv2-olympus-c8080wz.jpg'
            ,'exiv2-panasonic-dmc-fz5.jpg'
            ,'exiv2-sony-dsc-w7.jpg'
            ,'exiv2-canon-eos-20d.jpg'
            ,'exiv2-canon-eos-d30.jpg'
            ,'exiv2-canon-powershot-a520.jpg'
            ,'exiv2-photoshop.psd'
        ]

        images_2 = [
            'exiv2-empty.jpg'
            ,'20031214_000043.jpg'
            ,'20000506_020544.jpg'
            ,'20040329_224245.jpg'
            ,'20010405_235039.jpg'
            ,'20030925_201850.jpg'
            ,'20001026_044550.jpg'
            ,'20030926_111535.jpg'
            ,'20040316_075137.jpg'
            ,'20040208_093744.jpg'
            ,'20050218_212016.jpg'
            ,'20050527_051833.jpg'
            ,'20060802_095200.jpg'
            ,'20001004_015404.jpg'
            ,'20060127_225027.jpg'
            ,'20110626_213900.psd'
        ]

        images_3 = [
            'exiv2-empty.exv'
            ,'20031214_000043.exv'
            ,'20000506_020544.exv'
            ,'20040329_224245.exv'
            ,'20010405_235039.exv'
            ,'20030925_201850.exv'
            ,'20001026_044550.exv'
            ,'20030926_111535.exv'
            ,'20040316_075137.exv'
            ,'20040208_093744.exv'
            ,'20050218_212016.exv'
            ,'20050527_051833.exv'
            ,'20060802_095200.exv'
            ,'20001004_015404.exv'
            ,'20060127_225027.exv'
            ,'20110626_213900.exv'
        ]

        images_1_str = ' '.join(images_1)
        images_2_str = ' '.join(images_2)
        images_3_str = ' '.join(images_3)

        for i in images_1 + ['20110626_213900.psd']:
            BT.copyTestFile(i)

        out  = BT.Output()
        out += 'Exiv2 test directory -----------------------------------------------------'
        out += 'tmp/'

        out += '\nExiv2 version ------------------------------------------------------------'
        # Tweak this to avoid a maintenance headache with test/data/exiv2-test.out
        out += re.sub(r'exiv2.*', 'exiv2 0.27.0.0 (__ bit build)', BT.Executer('exiv2 -u -V').stdout)

        out += '\nExiv2 help ---------------------------------------------------------------'
        out += BT.Executer('exiv2 -u -h')

        out += '\n\nAdjust -------------------------------------------------------------------'
        out += BT.Executer('exiv2 -u -v -a-12:01:01 adjust  {images_1_str}', vars(), assert_returncode=[253])

        out += '\nRename -------------------------------------------------------------------'
        out += BT.Executer('exiv2 -u -vf rename             {images_1_str}', vars(), assert_returncode=[253])

        out += '\nPrint --------------------------------------------------------------------'
        out += BT.Executer('exiv2 -u -v print               {images_2_str}', vars(), assert_returncode=[253])
        out += ''
        out += BT.Executer('exiv2 -u -v -b -pt print        {images_2_str}', vars())
        e    = BT.Executer('exiv2 -u -v -b -pt print        {images_2_str}', vars(), redirect_stderr_to_stdout=False, decode_output=False)
        BT.save(e.stdout, 'iii')
        out += e.stderr.decode()

        out += '\nExtract Exif data --------------------------------------------------------'
        out += BT.Executer('exiv2 -u -vf extract            {images_2_str}', vars())

        out += '\nExtract Thumbnail --------------------------------------------------------'
        out += BT.Executer('exiv2 -u -vf -et extract        {images_2_str}', vars(), assert_returncode=[253])
        e    = BT.Executer('exiv2 -u -v -b -pt print        {images_3_str}', vars(), redirect_stderr_to_stdout=False, decode_output=False)
        BT.save(e.stdout, 'jjj')
        out += e.stderr.decode()

        out += '\nCompare image data and extracted data ------------------------------------'
        out += BT.diff('iii', 'jjj')

        out += '\nDelete Thumbnail ---------------------------------------------------------'
        out += BT.Executer('exiv2 -u -v -dt delete          {images_2_str}', vars())
        out += BT.Executer('exiv2 -u -vf -et extract        {images_2_str}', vars(), assert_returncode=[253])

        out += '\nDelete Exif data ---------------------------------------------------------'
        out += BT.Executer('exiv2 -u -v delete              {images_2_str}', vars())
        out += BT.Executer('exiv2 -u -v print               {images_2_str}', vars(), assert_returncode=[253])

        out += '\nInsert Exif data ---------------------------------------------------------'
        out += BT.Executer('exiv2 -u -v insert              {images_2_str}', vars())
        e    = BT.Executer('exiv2 -u -v -b -pt print        {images_3_str}', vars(), redirect_stderr_to_stdout=False, decode_output=False)
        BT.save(e.stdout, 'kkk')
        out += e.stderr.decode()

        out += '\nCompare original and inserted image data ---------------------------------'
        out += BT.diff('iii', 'kkk')

        BT.reportTest('exiv2-test', out)


    def geotag_test(self):
        # Test driver for geotag
        jpg      = 'FurnaceCreekInn.jpg'
        gpx      = 'FurnaceCreekInn.gpx'
        for i in [jpg, gpx]:
            BT.copyTestFile(i)

        out      = BT.Output()
        out     += '--- show GPSInfo tags ---'
        out     += BT.Executer('exiv2 -pa --grep GPSInfo    {jpg}', vars())

        out     += '--- deleting the GPSInfo tags'
        for tag in BT.Executer('exiv2 -Pk --grep GPSInfo    {jpg}', vars()).stdout.split('\n'):
            tag  = tag.rstrip(' ')
            out += BT.Executer('exiv2 -M"del {tag}"         {jpg}', vars())
        out     += BT.Executer('exiv2 -pa --grep GPS        {jpg}', vars(), assert_returncode=[0, 1])

        out     += '--- run geotag ---'
        e        = BT.Executer('geotag -ascii -tz -8:00     {jpg} {gpx}', vars())
        out     += ' '.join(e.stdout.split('\n')[0].split(' ')[1:])

        out     += '--- show GPSInfo tags ---'
        out     += BT.Executer('exiv2 -pa --grep GPSInfo    {jpg}', vars())

        BT.reportTest('geotag-test', out)


    def icc_test(self):
        # Test driver for exiv2.exe ICC support (-pS, -pC, -eC, -iC)

        def test1120(img):
            # --comment and -dc clobbered by writing ICC/JPG
            out      = BT.Output()
            if img  == 'Reagan2.jp2':
                return
            if img  == 'exiv2-bug1199.webp':
                out += BT.Executer('exiv2 --comment abcdefg   {img}', vars(), assert_returncode=[0, 1])
                out += BT.Executer('exiv2 -pS                 {img}', vars())
                out += ''
            else:
                out += BT.Executer('exiv2 --comment abcdefg   {img}', vars())
                out += BT.Executer('exiv2 -pS                 {img}', vars())
            out     += BT.Executer('exiv2 -pc                 {img}', vars())
            out     += BT.Executer('exiv2 -dc                 {img}', vars())
            out     += BT.Executer('exiv2 -pS                 {img}', vars())
            return str(out) or None

        # num = 1074  # ICC Profile Support
        out = BT.Output()
        for img in ['Reagan.jpg'
                    ,'exiv2-bug1199.webp'
                    ,'ReaganLargePng.png'
                    ,'ReaganLargeJpg.jpg'
                    ,'Reagan2.jp2'      # 1272 ReaganLargeTiff.tiff
                    ]:
            stub      = img.split('.')[0]
            iccname   = stub + '.icc'

            for i in ['large.icc', 'small.icc', img]:
                BT.copyTestFile(i)

            out      += BT.Executer('exiv2 -pS          {img}', vars())
            e         = BT.Executer('exiv2 -pC          {img}', vars(), compatible_output=False, decode_output=False)
            BT.save(e.stdout, stub + '_1.icc')
            out      += BT.Executer('exiv2 -eC --force  {img}', vars())
            BT.mv(iccname, stub + '_2.icc')
            out      += test1120(img)

            BT.copyTestFile('large.icc', iccname)
            out      += BT.Executer('exiv2 -iC          {img}', vars())
            e         = BT.Executer('exiv2 -pC          {img}', vars(), compatible_output=False, decode_output=False)
            BT.save(e.stdout, stub + '_large_1.icc')
            out      += BT.Executer('exiv2 -pS          {img}', vars())
            out      += BT.Executer('exiv2 -eC --force  {img}', vars())
            BT.mv(iccname, stub + '_large_2.icc')
            out      += test1120(img)

            BT.copyTestFile('small.icc', iccname)
            out      += BT.Executer('exiv2 -iC          {img}', vars())
            e         = BT.Executer('exiv2 -pC          {img}', vars(), compatible_output=False, decode_output=False)
            BT.save(e.stdout, stub + '_small_1.icc')
            out      += BT.Executer('exiv2 -pS          {img}', vars())
            out      += BT.Executer('exiv2 -eC --force  {img}', vars())
            BT.mv(iccname, stub + '_small_2.icc')
            out      += test1120(img)

            for f in [stub, stub + '_small', stub + '_large']:
                for i in [1, 2]:
                    out += BT.md5sum('{}_{}.icc'.format(f, i))

        BT.reportTest('icc-test', out)


    def image_test(self):
        test_files = ['table.jpg', 'smiley1.jpg', 'smiley2.jpg']
        erase_test_files = [
            'glider.exv'
            ,'iptc-noAPP13.jpg'
            ,'iptc-psAPP13-noIPTC.jpg'
            ,'iptc-psAPP13-noIPTC-psAPP13-wIPTC.jpg'
            ,'iptc-psAPP13s-noIPTC-psAPP13s-wIPTC.jpg'
            ,'iptc-psAPP13s-wIPTC-psAPP13s-noIPTC.jpg'
            ,'iptc-psAPP13s-wIPTCs-psAPP13s-wIPTCs.jpg'
            ,'iptc-psAPP13-wIPTC1-psAPP13-wIPTC2.jpg'
            ,'iptc-psAPP13-wIPTCbeg.jpg'
            ,'iptc-psAPP13-wIPTCempty.jpg'
            ,'iptc-psAPP13-wIPTCempty-psAPP13-wIPTC.jpg'
            ,'iptc-psAPP13-wIPTCend.jpg'
            ,'iptc-psAPP13-wIPTCmid1-wIPTCempty-wIPTCmid2.jpg'
            ,'iptc-psAPP13-wIPTCmid.jpg'
            ,'iptc-psAPP13-wIPTC-psAPP13-noIPTC.jpg'
        ]

        pass_count  = 0
        fail_count  = 0
        out = BT.Output()

        out += '\n--- Erase all tests ---'
        for i in test_files + erase_test_files:
            if BT.eraseTest(i):
                pass_count += 1
            else:
                fail_count += 1
                out        += 'Failed: ' + i

        out += '\n--- Copy all tests ---'
        for num, src in enumerate(test_files, 1):
            for dst in test_files:
                if BT.copyTest(num, src, dst):
                    pass_count += 1
                else:
                    fail_count += 1
                    out        += 'Failed: {}'.format((num, src, dst))

        out += '\n--- Copy iptc tests ---'
        for num, src in enumerate(test_files, 1):
            for dst in test_files:
                if BT.iptcTest(num, src, dst):
                    pass_count += 1
                else:
                    fail_count += 1
                    out        += 'Failed: {}'.format((num, src, dst))

        out += '\n--------------------\n'
        out += '{} passed, {} failed\n'.format(pass_count, fail_count)
        if fail_count:
            raise RuntimeError(str(out) + '\n' + BT.log.to_str())


    def io_test(self):
        # Test driver for file i/o
        test_files  = ['table.jpg', 'smiley2.jpg', 'ext.dat']
        for f in test_files:
            BT.copyTestFile(f)
            BT.ioTest(f)

        # Test http I/O
        def sniff(*files):
            result  = [str(os.path.getsize(i)) for i in files]
            result += [BT.md5sum(i) for i in files]
            return ' '.join(result)

        exiv2_http  = BT.Config.exiv2_http
        exiv2_port  = BT.Config.exiv2_port
        if not exiv2_http or not exiv2_port:
            print('Skipped http test. Because of invalid environment variables: EXIV2_HTTP={} EXIV2_PORT={}'.format(
                  exiv2_http, exiv2_port))
            return
        server_url  = '{}:{}'.format(exiv2_http,
                                     exiv2_port)
        server      = BT.HttpServer(bind=exiv2_http.lstrip('http://'),
                                    port=exiv2_port,    # It can be of type int or str
                                    work_dir=BT.Config.data_dir)
        try:
            server.start()
            out          = BT.Output()
            for img in ['table.jpg', 'Reagan.tiff', 'exiv2-bug922a.jpg']:
                files    = ['s0', 's1', 's2', '{}/{}'.format(server_url, img)]
                out     += BT.Executer('iotest ' + ' '.join(files))
                for f in files:
                    out += BT.Executer('exiv2 -g City -g DateTime {f}', vars())

            for num in ['0', '10', '1000']:
                out     += BT.Executer('iotest s0 s1 s2 {server_url}/table.jpg {num}', vars())
                out     += sniff('s0', 's1', 's2', os.path.join(BT.Config.data_dir, 'table.jpg'))

        except Exception as e:
            BT.log.error(e)
            raise RuntimeError('\n' + BT.log.to_str())

        finally:
            server.stop()   # While you're debugging, you can comment this line to keep the server running
            pass

        BT.reportTest('iotest', out)


    def iptc_test(self):
        # Test driver for Iptc metadata
        test_files = [
            'glider.exv'
            ,'iptc-noAPP13.jpg'
            ,'iptc-psAPP13-noIPTC.jpg'
            ,'iptc-psAPP13-noIPTC-psAPP13-wIPTC.jpg'
            ,'iptc-psAPP13s-noIPTC-psAPP13s-wIPTC.jpg'
            ,'iptc-psAPP13s-wIPTC-psAPP13s-noIPTC.jpg'
            ,'iptc-psAPP13s-wIPTCs-psAPP13s-wIPTCs.jpg'
            ,'iptc-psAPP13-wIPTC1-psAPP13-wIPTC2.jpg'
            ,'iptc-psAPP13-wIPTCbeg.jpg'
            ,'iptc-psAPP13-wIPTCempty.jpg'
            ,'iptc-psAPP13-wIPTCempty-psAPP13-wIPTC.jpg'
            ,'iptc-psAPP13-wIPTCend.jpg'
            ,'iptc-psAPP13-wIPTCmid1-wIPTCempty-wIPTCmid2.jpg'
            ,'iptc-psAPP13-wIPTCmid.jpg'
            ,'iptc-psAPP13-wIPTC-psAPP13-noIPTC.jpg'
            ,'smiley1.jpg'
            ,'smiley2.jpg'
            ,'table.jpg'
        ]

        pass_count  = 0
        fail_count  = 0
        out = BT.Output()

        out += '\n--- Read tests ---'
        for i in test_files:
            if BT.printTest(i):
                pass_count += 1
            else:
                fail_count += 1
                out        += 'Failed: ' + i

        out += '\n--- Remove tests ---'
        for i in test_files:
            if BT.removeTest(i):
                pass_count += 1
            else:
                fail_count += 1
                out        += 'Failed: ' + i

        out += '\n--- Add/Mod tests ---'
        for i in test_files:
            if BT.addModTest(i):
                pass_count += 1
            else:
                fail_count += 1
                out        += 'Failed: ' + i

        out += '\n--- Extended tests ---'
        for i in test_files:
            if BT.extendedTest(i):
                pass_count += 1
            else:
                fail_count += 1
                out        += 'Failed: ' + i

        out += '\n--------------------\n'
        out += '{} passed, {} failed\n'.format(pass_count, fail_count)
        if fail_count:
            raise RuntimeError(str(out) + '\n' + BT.log.to_str())


    def iso65k_test(self):
        # test for ISOs which follow Annex G of EXIF 2.3 spec, i.e. ISOs,
        # which cannot be represented by Exif.Photo.ISOSpeedRatings due to
        # being larger than 65k
        out      = BT.Output()

        # Checks for old way of ISO readout based on the 16bit value
        # input:
        # - Exif.Photo.ISOSpeedRatings being set to something <65k
        # output:
        # - value of Exif.Photo.ISOSpeedRatings
        num      = '0001'
        filename = 'exiv2-iso65k-{}.jpg'.format(num)
        BT.copyTestFile('exiv2-empty.jpg', filename)
        out     += '------> iso65k test {} <-------'.format(num)
        out     += BT.Executer("exiv2 -M'set Exif.Photo.ISOSpeedRatings 60001' {filename}", vars())
        out     += BT.Executer("exiv2 -ps                                      {filename}", vars())
        out     += ''

        # Old ISO is read out first, so if it doesn't indicate that
        # some higher ISO is used, the 16bit value should be returned,
        # ignoring the other tags (for now)
        # input:
        # - Exif.Photo.ISOSpeedRatings being set to something <65k
        # - Exif.Photo.SensitivityType being set to "REI"
        # - Exif.Photo.RecommendedExposureIndex being set to != ISOSpeedRatings
        # output:
        # - value of Exif.Photo.ISOSpeedRatings
        num      = '0002'
        filename = 'exiv2-iso65k-{}.jpg'.format(num)
        BT.copyTestFile('exiv2-empty.jpg', filename)
        out     += '------> iso65k test {} <-------'.format(num)
        out     += BT.Executer("exiv2 -M'set Exif.Photo.ISOSpeedRatings 60002'           {filename}", vars())
        out     += BT.Executer("exiv2 -M'set Exif.Photo.SensitivityType 2'               {filename}", vars())
        out     += BT.Executer("exiv2 -M'set Exif.Photo.RecommendedExposureIndex 444444' {filename}", vars())
        out     += BT.Executer("exiv2 -ps                                                {filename}", vars())
        out     += ''

        # Corner case check (highest ISO value not indicating possible
        # 16bit overflow in ISO)
        # input:
        # - Exif.Photo.ISOSpeedRatings being set to 65534
        # output:
        # - value of Exif.Photo.ISOSpeedRatings
        num      = '0003'
        filename = 'exiv2-iso65k-{}.jpg'.format(num)
        BT.copyTestFile('exiv2-empty.jpg', filename)
        out     += '------> iso65k test {} <-------'.format(num)
        out     += BT.Executer("exiv2 -M'set Exif.Photo.ISOSpeedRatings 65534' {filename}", vars())
        out     += BT.Executer("exiv2 -ps                                      {filename}", vars())
        out     += ''

        # Corner case check (ISO value indicating possible overflow,
        # but no additional informations available)
        # input:
        # - Exif.Photo.ISOSpeedRatings being set to 65535
        # - Exif.Photo.SensitivityType NOT SET
        # output:
        # - value of Exif.Photo.ISOSpeedRatings
        num      = '0004'
        filename = 'exiv2-iso65k-{}.jpg'.format(num)
        BT.copyTestFile('exiv2-empty.jpg', filename)
        out     += '------> iso65k test {} <-------'.format(num)
        out     += BT.Executer("exiv2 -M'set Exif.Photo.ISOSpeedRatings 65535' {filename}", vars())
        out     += BT.Executer("exiv2 -ps                                      {filename}", vars())
        out     += ''

        # possible ISO value overflow, but additional information not valid
        # input:
        # - Exif.Photo.ISOSpeedRatings being set to 65535
        # - Exif.Photo.SensitivityType being set to 0
        # output:
        # - value of Exif.Photo.ISOSpeedRatings
        num      = '0005'
        filename = 'exiv2-iso65k-{}.jpg'.format(num)
        BT.copyTestFile('exiv2-empty.jpg', filename)
        out     += '------> iso65k test {} <-------'.format(num)
        out     += BT.Executer("exiv2 -M'set Exif.Photo.ISOSpeedRatings 65535' {filename}", vars())
        out     += BT.Executer("exiv2 -M'set Exif.Photo.SensitivityType 0'     {filename}", vars())
        out     += BT.Executer("exiv2 -ps                                      {filename}", vars())
        out     += ''

        # possible ISO value overflow, but additional information not valid
        # input:
        # - Exif.Photo.ISOSpeedRatings being set to 65535
        # - Exif.Photo.SensitivityType being set to 8
        # output:
        # - value of Exif.Photo.ISOSpeedRatings
        num      = '0006'
        filename = 'exiv2-iso65k-{}.jpg'.format(num)
        BT.copyTestFile('exiv2-empty.jpg', filename)
        out     += '------> iso65k test {} <-------'.format(num)
        out     += BT.Executer("exiv2 -M'set Exif.Photo.ISOSpeedRatings 65535' {filename}", vars())
        out     += BT.Executer("exiv2 -M'set Exif.Photo.SensitivityType 8'     {filename}", vars())
        out     += BT.Executer("exiv2 -ps                                      {filename}", vars())
        out     += ''

        # possible ISO value overflow, but additional information partially valid
        # input:
        # - Exif.Photo.ISOSpeedRatings being set to 65535
        # - Exif.Photo.SensitivityType being set to 2 ("REI")
        # - Exif.Photo.RecommendedExposureIndex NOT SET
        # output:
        # - value of Exif.Photo.ISOSpeedRatings
        num      = '0007'
        filename = 'exiv2-iso65k-{}.jpg'.format(num)
        BT.copyTestFile('exiv2-empty.jpg', filename)
        out     += '------> iso65k test {} <-------'.format(num)
        out     += BT.Executer("exiv2 -M'set Exif.Photo.ISOSpeedRatings 65535' {filename}", vars())
        out     += BT.Executer("exiv2 -M'set Exif.Photo.SensitivityType 2'     {filename}", vars())
        out     += BT.Executer("exiv2 -ps                                      {filename}", vars())
        out     += ''

        # ISO value overflow, REI contains same value as 16bit ISO, though
        # input:
        # - Exif.Photo.ISOSpeedRatings being set to 65535
        # - Exif.Photo.SensitivityType being set to 2 ("REI")
        # - Exif.Photo.RecommendedExposureIndex set to 65530
        # output:
        # - value of Exif.Photo.RecommendedExposureIndex
        num      = '0008'
        filename = 'exiv2-iso65k-{}.jpg'.format(num)
        BT.copyTestFile('exiv2-empty.jpg', filename)
        out     += '------> iso65k test {} <-------'.format(num)
        out     += BT.Executer("exiv2 -M'set Exif.Photo.ISOSpeedRatings 65535'           {filename}", vars())
        out     += BT.Executer("exiv2 -M'set Exif.Photo.SensitivityType 2'               {filename}", vars())
        out     += BT.Executer("exiv2 -M'set Exif.Photo.RecommendedExposureIndex 65530'  {filename}", vars())
        out     += BT.Executer("exiv2 -ps                                                {filename}", vars())
        out     += ''

        # ISO value overflow, REI contains 16bit ISO value +1
        # input:
        # - Exif.Photo.ISOSpeedRatings being set to 65535
        # - Exif.Photo.SensitivityType being set to 2 ("REI")
        # - Exif.Photo.RecommendedExposureIndex set to 65536
        # output:
        # - value of Exif.Photo.RecommendedExposureIndex
        num      = '0009'
        filename = 'exiv2-iso65k-{}.jpg'.format(num)
        BT.copyTestFile('exiv2-empty.jpg', filename)
        out     += '------> iso65k test {} <-------'.format(num)
        out     += BT.Executer("exiv2 -M'set Exif.Photo.ISOSpeedRatings 65535'           {filename}", vars())
        out     += BT.Executer("exiv2 -M'set Exif.Photo.SensitivityType 2'               {filename}", vars())
        out     += BT.Executer("exiv2 -M'set Exif.Photo.RecommendedExposureIndex 65536'  {filename}", vars())
        out     += BT.Executer("exiv2 -ps                                                {filename}", vars())
        out     += ''

        # old ISO not set
        # input:
        # - Exif.Photo.ISOSpeedRatings is NOT SET
        # - Exif.Photo.SensitivityType being set to 2 ("REI")
        # - Exif.Photo.RecommendedExposureIndex set to <65k
        # output:
        # - value of Exif.Photo.RecommendedExposureIndex
        num      = '0010'
        filename = 'exiv2-iso65k-{}.jpg'.format(num)
        BT.copyTestFile('exiv2-empty.jpg', filename)
        out     += '------> iso65k test {} <-------'.format(num)
        out     += BT.Executer("exiv2 -M'set Exif.Photo.SensitivityType 2'               {filename}", vars())
        out     += BT.Executer("exiv2 -M'set Exif.Photo.RecommendedExposureIndex 60010'  {filename}", vars())
        out     += BT.Executer("exiv2 -ps                                                {filename}", vars())
        out     += ''

        # old ISO not set
        # input:
        # - Exif.Photo.ISOSpeedRatings is NOT SET
        # - Exif.Photo.SensitivityType being set to 2 ("REI")
        # - Exif.Photo.RecommendedExposureIndex set to >65k
        # output:
        # - value of Exif.Photo.RecommendedExposureIndex
        num      = '0011'
        filename = 'exiv2-iso65k-{}.jpg'.format(num)
        BT.copyTestFile('exiv2-empty.jpg', filename)
        out     += '------> iso65k test {} <-------'.format(num)
        out     += BT.Executer("exiv2 -M'set Exif.Photo.SensitivityType 2'               {filename}", vars())
        out     += BT.Executer("exiv2 -M'set Exif.Photo.RecommendedExposureIndex 100011' {filename}", vars())
        out     += BT.Executer("exiv2 -ps                                                {filename}", vars())
        out     += ''

        BT.reportTest('iso65k-test', out)


    def modify_test(self):
        # Test driver for write unit tests to build Exif metadata from scratch
        for i in ['exiv2-empty.jpg', 'exiv2-gc.jpg', 'modifycmd1.txt', 'modifycmd2.txt']:
            BT.copyTestFile(i)
        out  = BT.Output()
        out += BT.Executer('exiv2 -v -m modifycmd1.txt exiv2-empty.jpg')
        out += BT.Executer('exiv2 -v -m modifycmd2.txt exiv2-gc.jpg')
        out += BT.Executer('exiv2 -v -pi exiv2-empty.jpg')
        out += BT.Executer('exiv2 -v -pt exiv2-empty.jpg exiv2-gc.jpg')
        BT.reportTest('modify-test', out)


    def nls_test(self):
        # Test driver for exiv2.exe nls support
        nls          = BT.Executer('exiv2 -vVg nls').stdout.split('\n')[1]
        if nls      != 'enable_nls=1':
            print('Skipped. Because exiv2 is not built with nls.')
            return

        if BT.Config.system_name == 'Windows':
            print('Skipped. Because nls_test is not supported on Windows.')
            return

        if BT.Config.system_name == 'Linux':
            LANG = 'LANGUAGE'
        else:
            LANG = 'LANG'

        share_dir    = os.path.normpath(os.path.join(BT.Config.bin_dir, '..', 'share2'))
        os.makedirs(share_dir, exist_ok=True)
        locale_dir   = '/usr/local/share/locale'
        if os.path.isdir(locale_dir) and os.path.isdir(share_dir):
            BT.cp(locale_dir, share_dir)
        else:
            print('Skipped. Because localisation files are not installed in {}.'.format(locale_dir))

        # The above part is checking the environment, and the following part is executing the actual test
        out      = BT.Output()
        for language in ['fr_FR', 'es_ES']:
            e    = BT.Executer(cmd='exiv2', extra_env={'LC_ALL': language, LANG: language}, assert_returncode=[1], decode_output=False)
            out += e.stdout.split(b'\n')[0].decode()
        BT.reportTest('nls-test', out,forgive=True)


    def path_test(self):
        # Mini test-driver for path utility functions
        BT.copyTestFile('path-test.txt')
        out      = BT.Output()
        out     += BT.Executer('path-test path-test.txt')
        # print(out)


    def png_test(self):
        # Test PNG support (-pS, -pc, -c, -pa)
        out   = BT.Output()
        files = ['1343_empty.png'
                ,'1343_comment.png'
                ,'1343_exif.png'
                ]
        for file in files:
            BT.copyTestFile(file)
            for i in [1,2]:
                out  += BT.Executer('exiv2 -pS    {file}', vars())
                out  += BT.Executer('exiv2 -pc    {file}', vars())
                if i ==1:
                  out+= ''
                out  += BT.Executer('exiv2 -pa    {file}', vars())
                out  += BT.Executer('exiv2 -c "changed comment" {file}', vars())

        BT.reportTest('png-test', out) # skip!  blank lines from -pc are in the wrong order!


    def preview_test(self):
        # Test driver for previews
        images = [
            'exiv2-bug443.jpg'
            ,'exiv2-bug444.jpg'
            ,'exiv2-bug445.jpg'
            ,'exiv2-bug447.jpg'
            ,'exiv2-bug501.jpg'
            ,'exiv2-bug528.jpg'
            ,'exiv2-canon-eos-20d.jpg'
            ,'exiv2-canon-eos-300d.jpg'
            ,'exiv2-canon-eos-d30.jpg'
            ,'exiv2-canon-powershot-a520.jpg'
            ,'exiv2-canon-powershot-s40.crw'
            ,'exiv2-fujifilm-finepix-s2pro.jpg'
            ,'exiv2-gc.jpg'
            ,'exiv2-kodak-dc210.jpg'
            ,'exiv2-nikon-d70.jpg'
            ,'exiv2-nikon-e950.jpg'
            ,'exiv2-nikon-e990.jpg'
            ,'exiv2-olympus-c8080wz.jpg'
            ,'exiv2-panasonic-dmc-fz5.jpg'
            ,'exiv2-photoshop.psd'
            ,'exiv2-pre-in-xmp.xmp'
            ,'exiv2-sigma-d10.jpg'
            ,'exiv2-sony-dsc-w7.jpg'
            ,'glider.exv'
            ,'imagemagick.pgf'
            ,'iptc-psAPP13-noIPTC-psAPP13-wIPTC.jpg'
            ,'iptc-psAPP13-noIPTC.jpg'
            ,'iptc-psAPP13-wIPTC-psAPP13-noIPTC.jpg'
            ,'iptc-psAPP13-wIPTC1-psAPP13-wIPTC2.jpg'
            ,'iptc-psAPP13-wIPTCbeg.jpg'
            ,'iptc-psAPP13-wIPTCempty-psAPP13-wIPTC.jpg'
            ,'iptc-psAPP13-wIPTCempty.jpg'
            ,'iptc-psAPP13-wIPTCend.jpg'
            ,'iptc-psAPP13-wIPTCmid.jpg'
            ,'iptc-psAPP13-wIPTCmid1-wIPTCempty-wIPTCmid2.jpg'
            ,'smiley2.jpg'
        ]
        out         = BT.Output()
        report      = BT.Output()
        pass_count  = 0
        fail_count  = 0
        preview_dir = os.path.join(BT.Config.data_dir, 'preview')

        for filename in images:
            image   = filename.split('.')[0]
            BT.copyTestFile(filename)
            out    += '\n-----> {} <-----\n'.format(filename)

            out    += 'Command: exiv2 -pp ' + filename
            e       = BT.Executer('exiv2 -pp {filename}', vars(), assert_returncode=None, redirect_stderr_to_stdout=False)
            out    += e.stdout
            out    += 'Exit code: {}'.format(e.returncode)
            BT.rm(*BT.find(pattern=image + '-preview*'))

            out    += '\nCommand: exiv2 -f -ep ' + filename
            e       = BT.Executer('exiv2 -f -ep {filename}', vars(), assert_returncode=None, redirect_stderr_to_stdout=False)
            out    += e.stdout
            out    += 'Exit code: {}'.format(e.returncode)

            # Check the difference
            e       = BT.Executer('exiv2 -pp {filename}', vars(), assert_returncode=None, redirect_stderr_to_stdout=False)
            preview_num         = e.stdout[:e.stdout.find(':')].lstrip('Preview ')
            for test_file in BT.find(pattern='{image}-preview{preview_num}.*'.format(**vars())):
                reference_file  = os.path.join(preview_dir, test_file)
                if BT.diffCheck(reference_file, test_file, in_bytes=True):
                    pass_count += 1
                else:
                    fail_count += 1
                    report     += 'Failed: ' + filename

        report  += '\n{} passed, {} failed\n'.format(pass_count, fail_count)
        if fail_count:
            raise RuntimeError('\n' + str(report) + '\n' + BT.log.to_str())

        BT.reportTest('preview-test', out)


    def stdin_test(self):
        return # temporarily disable
        # Test driver for stdin
        try:
            import lxml
        except ModuleNotFoundError:
            print('Skipped. Because it misses module lxml. Please install: `pip install lxml`')
            return

        out     = BT.Output()
        a       = 'exiv2-bug1229.jpg'  # jpg with 2 APP1/xap segments
        b       = 'girl.jpg'
        BT.copyTestFile(a)

        def get_xmpData(img):
            e    = BT.Executer('exiv2 -pX   {img}', vars(), decode_output=False)
            return e.stdout.replace(b'\n', b'\r\n', 16) # Ignore the difference in newline

        BT.copyTestFile(a, b)
        out     += BT.Executer('exiv2 -pS   {b}', vars())
        out     += BT.Executer('exiv2 -dX   {b}', vars())  # remove first
        out     += BT.Executer('exiv2 -pS   {b}', vars())

        e = BT.Executer('exiv2 -pX   {a}', vars(), decode_output=False)
        with open('out2', 'wb') as f:
            f.write(e.stdout)

        out     += BT.Executer('exiv2 -iXX- {b}', vars(), stdin=get_xmpData(a))
        out     += BT.Executer('exiv2 -pS   {b}', vars())

        BT.copyTestFile(a, b)
        out     += BT.Executer('exiv2 -dX   {b}', vars())
        out     += BT.Executer('exiv2 -dX   {b}', vars())
        out     += BT.Executer('exiv2 -pS   {b}', vars())
        out     += BT.Executer('exiv2 -iXX- {b}', vars(), stdin=get_xmpData(a))
        out     += BT.Executer('exiv2 -pS   {b}', vars())

        for f in ['Reagan.jpg', 'Reagan.tiff', 'ReaganSmallPng.png']:
            BT.copyTestFile(f)
            out     += BT.Executer('exiv2 -iXX- {f}', vars(), stdin=get_xmpData(a))
            e        = BT.Executer('exiv2 -pX   {f}', vars())
            out     += """
<?xml version="1.0"?>
<?xpacket begin='\ufeff' id='W5M0MpCehiHzreSzNTczkc9d'?>
{}
<?xpacket end='w'?>
""".strip('\n').format(BT.pretty_xml(e.stdout))

        for f in ['Reagan.jpg', 'ReaganSmallPng.png']:
            BT.copyTestFile(f)
            BT.copyTestFile(a, b)
            out     += BT.Executer('exiv2 -pS   {b}', vars())
            e        = BT.Executer('exiv2 -ea-  {f}', vars(), decode_output=False)
            out     += BT.Executer('exiv2 -ia-  {b}', vars(), stdin=e.stdout)
            out     += BT.Executer('exiv2 -pS   {b}', vars())

        BT.copyTestFile('Reagan.tiff')   # 1272 ReaganLargeTiff.tiff
        for img in ['Reagan.jpg', 'ReaganSmallPng.png', 'exiv2-bug1199.webp']:
            BT.copyTestFile(img)
            e        = BT.Executer('exiv2 -eC-  Reagan.tiff', decode_output=False)

            # Ignore the difference in the path separator
            stdout = e.stdout
            for pair in [
                (b'\x03/\x9e', b'\x03\\\x9e'),
                (b'\x0c/\x0c', b'\x0c\\\x0c'),
                (b'V/V'      , b'V\\V'      ),
                (b'\xe5/5'   , b'\xe5\\5'   ),
                (b'5/\x86'   , b'5\\\x86'   ),
                (b'\x86/\xd6', b'\x86\\\xd6'),
                (b'\xac/\xac', b'\xac\\\xac'),
                (b'\xd7/\xd7', b'\xd7\\\xd7'),
            ]:
                stdout  = stdout.replace(pair[0], pair[1])

            out     += BT.Executer('exiv2 -iC-  {img}', vars(), stdin=stdout)
            out     += BT.Executer('exiv2 -pS   {img}', vars())
            if img == 'ReaganSmallPng.png':
                with open('out2', 'wb') as f:
                    f.write(e.stdout)

        BT.reportTest('stdin-test', out)


    def stringto_test(self):
        # Test driver for tests of stringToLong/Float/Rational
        out      = BT.Output()
        out     += BT.Executer('stringto-test')
        BT.reportTest('stringto-test', out)


    def tiff_test(self):
        # TIFF parser test driver

        def exifprobe(img):
            out  = BT.Output()

            # Convert each line from that format:
            #   Exif.Image.NewSubfileType       Long    1  Primary image
            # to that format:
            #   NewSubfileType 	 Long 	 1 	 "Primary image"
            tags = []
            typs = []
            lens = []
            vals = []
            e    = BT.Executer('exiv2 -pa {img}', vars(), redirect_stderr_to_stdout=False)
            for line in e.stdout.split('\n'):
                fields = re.findall(r'^(\S*)\s*(\S*)\s*(\d*)\s*(.*)$', line)[0]
                tags  += [fields[0].split('.')[2]]      # tag
                typs  += [fields[1]]                    # type
                lens  += [fields[2]]                    # length
                vals  += [fields[3]]                    # value

            out += 'exiv2 -pa output'
            for i in range(len(tags)):
                out += ' \t '.join([tags[i], typs[i], lens[i], '"{}"'.format(vals[i])])

            # Parse -pS output:
            # address |    tag                           |      type |    count |   offset | value
            #     254 | 0x00fe NewSubfileType            |      LONG |        1 |        0 | 0
            TAGS = []
            TYPS = []
            LENS = []
            VALS = []
            e    = BT.Executer('exiv2 -pS {img}', vars(), redirect_stderr_to_stdout=False)
            for line in e.stdout.split('\n')[2:-1]:
                fields = [i.strip(' ') for i in line.split('|')]
                TAGS  += [fields[1].split(' ')[1]]
                TYPS  += [fields[2]]
                LENS  += [fields[3]]
                VALS  += [fields[5]]

            out += '\nexiv2 -pS output'
            for i in range(len(TAGS)):
                out += ' \t '.join([TAGS[i], TYPS[i], LENS[i], '"{}"'.format(VALS[i])])

            out += '\nAnalysis'
            out += 'count =  {} COUNT =  {}'.format(len(tags), len(TAGS))

            # Make them have the same number of lines
            max_lines  = max(len(TAGS), len(tags))
            for _list in [tags, typs, lens, vals, TAGS, TYPS, LENS, VALS]:
                _list += [''] * (max_lines - len(_list))

            # Compare the main fields of each line
            for i in range(max_lines):
                if TAGS[i]  != tags[i]:
                    out     += 'TAG {} {} mismatch'.format(TAGS[i], tags[i]).replace('  ', ' ')

                TYPS[i]      = TYPS[i].upper()
                typs[i]      = typs[i].upper()
                if TYPS[i]  != typs[i]:
                    out     += 'TYPE {} {} mismatch'.format(TYPS[i], typs[i]).replace('  ', ' ')

                if LENS[i]  != lens[i]:
                    out     += 'Length {} {} mismatch'.format(LENS[i], lens[i]).replace('  ', ' ')

                if typs[i]  == 'ASCII' and VALS[i] != vals[i]:
                    out     += 'Value {} {} mismatch'.format(VALS[i], vals[i]).replace('  ', ' ')

            return str(out)

        test_file = 'mini9.tif'
        BT.copyTestFile(test_file)
        out      = BT.Output()
        out     += exifprobe(test_file)
        out     += BT.Executer('tiff-test {test_file}', vars())
        out     += exifprobe(test_file)
        BT.reportTest('tiff-test', out)


    def version_test(self):
        # Test driver for exiv2 --verbose --version
        out      = BT.Output()
        out     += BT.Executer('exiv2 --verbose --version')


    def webp_test(self):
        return # temporarily disable

        # Test driver for webp
        webp     = 'exiv2-bug1199.webp' # http://dev.exiv2.org/attachments/download/1033/Stonehenge-with-icc.webp
        icc      = 'exiv2-bug1199.icc'
        exv      = 'exiv2-bug1199.exv'
        xmp      = 'exiv2-bug1199.xmp'
        tiff     = 'Reagan.tiff'
        out      = BT.Output()

        # Extract the XMP
        BT.copyTestFile(webp)
        out     += BT.Executer('exiv2 -pS {webp}', vars())
        e        = BT.Executer('exiv2 -pX {webp}', vars())
        out     += """
<?xml version="1.0"?>
<?xpacket begin="\ufeff" id="W5M0MpCehiHzreSzNTczkc9d"?>
{}
<?xpacket end="w"?>
""".strip('\n').format(BT.pretty_xml(e.stdout))

        # Test deleting metadata
        for option in ['-dC', '-de', '-dx', '-dCe', '-dCx', '-dCxe']:
            BT.copyTestFile(webp)
            out     += BT.Executer('exiv2 -pS      {webp}', vars())
            out     += BT.Executer('exiv2 {option} {webp}', vars())
            out     += BT.Executer('exiv2 -pS      {webp}', vars())

        # Extract the icc
        BT.copyTestFile(webp)
        BT.copyTestFile(tiff)
        out     += BT.Executer('exiv2 -pS {webp}', vars())
        BT.save(   BT.Executer('exiv2 -pC {tiff}', vars(), decode_output=False).stdout, icc)
        out     += BT.Executer('exiv2 -iC {webp}', vars())
        out     += BT.Executer('exiv2 -pS {webp}', vars())

        # Copy the XMP from the test file
        BT.copyTestFile(webp)
        BT.save(   BT.Executer('exiv2 -pX {webp}', vars(), decode_output=False).stdout, xmp)
        out     += BT.Executer('exiv2 -ea --force {webp}', vars())

        BT.copyTestFile(webp)
        out     += BT.Executer('exiv2 -pS  {webp}', vars())
        out     += BT.Executer('exiv2 -iXX {webp}', vars())
        out     += BT.Executer('exiv2 -pS  {webp}', vars())
        out     += BT.Executer('exiv2 -ix  {webp}', vars())

        # Copy the XMP from Reagan.tiff to test file
        BT.copyTestFile(tiff)
        BT.save(   BT.Executer('exiv2 -pX  {tiff}', vars(), decode_output=False).stdout, xmp)
        out     += BT.Executer('exiv2 -ea --force  {tiff}', vars())
        BT.mv('Reagan.exv', exv)

        BT.copyTestFile(webp)
        out     += BT.Executer('exiv2 -pS   {webp}', vars())
        out     += BT.Executer('exiv2 -iXX  {webp}', vars())
        out     += BT.Executer('exiv2 -pS   {webp}', vars())
        out     += BT.Executer('exiv2 -ix   {webp}', vars())

        # Copy the XMP from exiv2-bug922.jpg to test file
        BT.copyTestFile('exiv2-bug922.jpg')
        BT.save(   BT.Executer('exiv2 -pX  exiv2-bug922.jpg', decode_output=False).stdout, xmp)
        BT.Executer(           'exiv2 -ea --force  exiv2-bug922.jpg')
        BT.mv('exiv2-bug922.exv', exv)

        BT.copyTestFile(webp)
        out     += BT.Executer('exiv2 -pS   {webp}', vars())
        out     += BT.Executer('exiv2 -ix   {webp}', vars())
        out     += BT.Executer('exiv2 -pS   {webp}', vars())
        out     += BT.Executer('exiv2 -iXX  {webp}', vars())
        out     += BT.Executer('exiv2 -pS   {webp}', vars())

        BT.copyTestFile('exiv2-bug922.jpg', webp)
        out     += BT.Executer('exiv2 --force -ea  {webp}', vars())
        BT.copyTestFile(webp)
        out     += BT.Executer('exiv2 -pS   {webp}', vars())
        out     += BT.Executer('exiv2 -ie   {webp}', vars())
        out     += BT.Executer('exiv2 -pS   {webp}', vars())

        BT.reportTest('webp-test', out)


    def write_test(self):
        # Test driver for the write unit tests
        images = [
            'exiv2-canon-powershot-s40.jpg'
            ,'exiv2-kodak-dc210.jpg'
            ,'exiv2-fujifilm-finepix-s2pro.jpg'
            ,'exiv2-sigma-d10.jpg'
            ,'exiv2-nikon-e990.jpg'
            ,'exiv2-nikon-d70.jpg'
            ,'exiv2-nikon-e950.jpg'
        ]
        for i in images:
            BT.copyTestFile(i)

        out  = BT.Output()
        out += BT.runTestCase( 1, 'exiv2-canon-powershot-s40.jpg')
        out += BT.runTestCase( 2, 'exiv2-canon-powershot-s40.jpg')
        out += BT.runTestCase( 3, 'exiv2-kodak-dc210.jpg')
        out += BT.runTestCase( 4, 'exiv2-canon-powershot-s40.jpg')
        out += BT.runTestCase( 5, 'exiv2-canon-powershot-s40.jpg')
        out += BT.runTestCase( 6, 'exiv2-kodak-dc210.jpg')
        out += BT.runTestCase( 7, 'exiv2-fujifilm-finepix-s2pro.jpg')
        out += BT.runTestCase( 8, 'exiv2-sigma-d10.jpg')
        out += BT.runTestCase( 9, 'exiv2-nikon-e990.jpg')
        out += BT.runTestCase(10, 'exiv2-nikon-e950.jpg')
        out += BT.runTestCase(11, 'exiv2-nikon-d70.jpg')

        # Adjust the output to be compatible with the reference output
        out = str(out)
        for img in images:
            out = out.replace('Reading file ' + img, 'Reading file ./' + img)

        # Ignore output differences between BT.diff() and GNU dIff
        for pair in [
            ('24,2c24,2', '24,25c24,25'),
            ('29,2c29,2', '29,30c29,30'),
            ('27,2c27,2', '27,28c27,28'),
            ('28,2c28,2', '28,29c28,29'),
        ]:
            out  = out.replace(pair[0], pair[1])

        BT.reportTest('write-test', out)


    def write2_test(self):
        # Test driver for write unit tests to build Exif metadata from scratch
        img      = 'exiv2-empty.jpg'
        BT.copyTestFile(img)
        out      = BT.Output()
        out     += BT.Executer('write2-test {img}', vars())
        BT.reportTest('write2-test', out)


    def xmpparser_test(self):
        # XMP parser test driver
        images = ['BlueSquare.xmp', 'StaffPhotographer-Example.xmp', 'xmpsdk.xmp']
        out      = BT.Output()

        for img in images:
            BT.copyTestFile(img)
            out += BT.Executer('xmpparser-test {img}', vars())
            out += BT.diff(img, img + '-new')

        xmp = 'xmpsdk.xmp'
        BT.save(BT.Executer('xmpparse {xmp}'    , vars()).stdout, 't1')
        BT.save(BT.Executer('xmpparse {xmp}-new', vars()).stdout, 't2')
        out += BT.diff('t1', 't2')

        out += BT.Executer('xmpsample')
        for img in ['exiv2-empty.jpg', 'cmdxmp.txt']:
            BT.copyTestFile(img)
        out += BT.Executer('exiv2 -v -m cmdxmp.txt exiv2-empty.jpg', assert_returncode=[0, 1])
        out += BT.Executer('exiv2 -v -px exiv2-empty.jpg')

        # Ignore output differences between BT.diff() and GNU dIff
        out = str(out)
        out  = out.replace("""
34,0c35
---
>      <rdf:li xml:lang="x-default">Blue Square Test File - .jpg</rdf:li>
36c36,0
<      <rdf:li xml:lang="x-default">Blue Square Test File - .jpg</rdf:li>
67,21c67,21
""".strip('\n'),
"""
35d34
<      <rdf:li xml:lang="en-US">Blue Square Test File - .jpg</rdf:li>
36a36
>      <rdf:li xml:lang="en-US">Blue Square Test File - .jpg</rdf:li>
67,87c67,87
""".strip('\n'))
        for pair in [
            ('46,0c47\n---'             , '46a47'),
            ('160,32c161'               , '160,191c161'),
            ('1,49c1,65'                , '1,48c1,65'),
            ('<     </rdf:RDF>\n< '     , '<     </rdf:RDF>'),
            ('> <?xpacket end="w"?>\n'  , '> <?xpacket end="w"?>\n/ No newline at end of file'),
        ]:
            out  = out.replace(pair[0], pair[1])

        BT.reportTest('xmpparser-test', out)




# -*- coding: utf-8 -*-

import system_tests

class CheckRegularExpressionSupport(metaclass=system_tests.CaseMeta):

    url = "http://dev.exiv2.org/issues/1024"

    filename = system_tests.path("$data_path/exiv2-bug1024.exv")
    commands = [ "$exiv2 -pa --grep gpsl/i $filename" ]

    stdout = [ """Exif.GPSInfo.GPSLatitudeRef                  Ascii       2  North
Exif.GPSInfo.GPSLatitude                     Rational    3  52deg 4' 0"
Exif.GPSInfo.GPSLongitudeRef                 Ascii       2  East
Exif.GPSInfo.GPSLongitude                    Rational    3  1deg 14' 0"
"""
    ]
    stderr = [""]
    retval = [0]



# -*- coding: utf-8 -*-

from system_tests import CaseMeta, path


class WebPImageGetHeaderOffset(metaclass=CaseMeta):
    """
    Regression test for the bug described in:
    https://github.com/Exiv2/exiv2/pull/960
    """
    url = "https://github.com/Exiv2/exiv2/pull/960"

    filename1 = path("$data_path/issue_960.poc.webp")
    commands = ["$exiv2 $filename1"]
    stdout = [""]
    stderr = [
"""Warning: Failed to decode Exif metadata.
Exiv2 exception in print action for file $filename1:
$kerCorruptedMetadata
"""
]
    retval = [1]



# -*- coding: utf-8 -*-

from system_tests import DeleteFiles, CopyFiles, CaseMeta, path


@DeleteFiles("$xmpfile")
@CopyFiles("$data_path/exiv2-empty.jpg")
class WrongXmpTypeForNestedXmpKeys(metaclass=CaseMeta):

    url = "http://dev.exiv2.org/issues/$num"

    num = 799
    cmdfile = path("$data_path/bug$num.cmd")

    filename_common = path("$data_path/exiv2-empty_copy")
    filename = "$filename_common.jpg"
    xmpfile = "$filename_common.xmp"

    commands = [
        "$exiv2 -v -m $cmdfile $filename",
        "$exiv2 -v -pa $filename",
        "$exiv2 -f -eX $filename",
    ]

    def post_tests_hook(self):
        with open(self.xmpfile, "r", encoding='utf-8') as xmp_file:
            self.assertMultiLineEqual(self.xmp_packet, xmp_file.read(-1))

    xmp_packet = """<?xpacket begin="\ufeff" id="W5M0MpCehiHzreSzNTczkc9d"?>
<x:xmpmeta xmlns:x="adobe:ns:meta/" x:xmptk="XMP Core 4.4.0-Exiv2">
 <rdf:RDF xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#">
  <rdf:Description rdf:about=""
    xmlns:MP="http://ns.microsoft.com/photo/1.2/"
    xmlns:MPRI="http://ns.microsoft.com/photo/1.2/t/RegionInfo#"
    xmlns:MPReg="http://ns.microsoft.com/photo/1.2/t/Region#"
    xmlns:mwg-rs="http://www.metadataworkinggroup.com/schemas/regions/"
    xmlns:stDim="http://ns.adobe.com/xap/1.0/sType/Dimensions#"
    xmlns:stArea="http://ns.adobe.com/xmp/sType/Area#">
   <MP:RegionInfo rdf:parseType="Resource">
    <MPRI:Regions>
     <rdf:Bag>
      <rdf:li
       MPReg:Rectangle="0.11, 0.22, 0.33, 0.44"
       MPReg:PersonDisplayName="Baby Gnu"/>
     </rdf:Bag>
    </MPRI:Regions>
   </MP:RegionInfo>
   <mwg-rs:Regions rdf:parseType="Resource">
    <mwg-rs:AppliedToDimensions
     stDim:w="1600"
     stDim:h="800"
     stDim:unit="pixel"/>
    <mwg-rs:RegionList>
     <rdf:Bag>
      <rdf:li>
       <rdf:Description
        mwg-rs:Name="Baby Gnu"
        mwg-rs:Type="Face">
       <mwg-rs:Area
        stArea:x="0.275312"
        stArea:y="0.3775"
        stArea:w="0.164375"
        stArea:h="0.28125"
        stArea:unit="normalized"/>
       </rdf:Description>
      </rdf:li>
     </rdf:Bag>
    </mwg-rs:RegionList>
   </mwg-rs:Regions>
  </rdf:Description>
 </rdf:RDF>
</x:xmpmeta>
<?xpacket end="w"?>"""

    stdout = [
        """File 1/1: $filename
Set Xmp.MP.RegionInfo/MPRI:Regions "" (XmpBag)
Set Xmp.MP.RegionInfo/MPRI:Regions[1]/MPReg:Rectangle "0.11, 0.22, 0.33, 0.44" (XmpText)
Set Xmp.MP.RegionInfo/MPRI:Regions[1]/MPReg:PersonDisplayName "Baby Gnu" (XmpText)
Set Xmp.mwg-rs.Regions/mwg-rs:AppliedToDimensions/stDim:w "1600" (XmpText)
Set Xmp.mwg-rs.Regions/mwg-rs:AppliedToDimensions/stDim:h "800" (XmpText)
Set Xmp.mwg-rs.Regions/mwg-rs:AppliedToDimensions/stDim:unit "pixel" (XmpText)
Set Xmp.mwg-rs.Regions/mwg-rs:RegionList "" (XmpBag)
Set Xmp.mwg-rs.Regions/mwg-rs:RegionList[1]/mwg-rs:Name "Baby Gnu" (XmpText)
Set Xmp.mwg-rs.Regions/mwg-rs:RegionList[1]/mwg-rs:Type "Face" (XmpText)
Set Xmp.mwg-rs.Regions/mwg-rs:RegionList[1]/mwg-rs:Area/stArea:x "0.275312" (XmpText)
Set Xmp.mwg-rs.Regions/mwg-rs:RegionList[1]/mwg-rs:Area/stArea:y "0.3775" (XmpText)
Set Xmp.mwg-rs.Regions/mwg-rs:RegionList[1]/mwg-rs:Area/stArea:w "0.164375" (XmpText)
Set Xmp.mwg-rs.Regions/mwg-rs:RegionList[1]/mwg-rs:Area/stArea:h "0.28125" (XmpText)
Set Xmp.mwg-rs.Regions/mwg-rs:RegionList[1]/mwg-rs:Area/stArea:unit "normalized" (XmpText)
""",
        """File 1/1: $filename
Xmp.MP.RegionInfo                            XmpText     0  type="Struct"
Xmp.MP.RegionInfo/MPRI:Regions               XmpText     0  type="Bag"
Xmp.MP.RegionInfo/MPRI:Regions[1]            XmpText     0  type="Struct"
Xmp.MP.RegionInfo/MPRI:Regions[1]/MPReg:Rectangle XmpText    22  0.11, 0.22, 0.33, 0.44
Xmp.MP.RegionInfo/MPRI:Regions[1]/MPReg:PersonDisplayName XmpText     8  Baby Gnu
Xmp.mwg-rs.Regions                           XmpText     0  type="Struct"
Xmp.mwg-rs.Regions/mwg-rs:AppliedToDimensions XmpText     0  type="Struct"
Xmp.mwg-rs.Regions/mwg-rs:AppliedToDimensions/stDim:w XmpText     4  1600
Xmp.mwg-rs.Regions/mwg-rs:AppliedToDimensions/stDim:h XmpText     3  800
Xmp.mwg-rs.Regions/mwg-rs:AppliedToDimensions/stDim:unit XmpText     5  pixel
Xmp.mwg-rs.Regions/mwg-rs:RegionList         XmpText     0  type="Bag"
Xmp.mwg-rs.Regions/mwg-rs:RegionList[1]      XmpText     0  type="Struct"
Xmp.mwg-rs.Regions/mwg-rs:RegionList[1]/mwg-rs:Name XmpText     8  Baby Gnu
Xmp.mwg-rs.Regions/mwg-rs:RegionList[1]/mwg-rs:Type XmpText     4  Face
Xmp.mwg-rs.Regions/mwg-rs:RegionList[1]/mwg-rs:Area XmpText     0  type="Struct"
Xmp.mwg-rs.Regions/mwg-rs:RegionList[1]/mwg-rs:Area/stArea:x XmpText     8  0.275312
Xmp.mwg-rs.Regions/mwg-rs:RegionList[1]/mwg-rs:Area/stArea:y XmpText     6  0.3775
Xmp.mwg-rs.Regions/mwg-rs:RegionList[1]/mwg-rs:Area/stArea:w XmpText     8  0.164375
Xmp.mwg-rs.Regions/mwg-rs:RegionList[1]/mwg-rs:Area/stArea:h XmpText     7  0.28125
Xmp.mwg-rs.Regions/mwg-rs:RegionList[1]/mwg-rs:Area/stArea:unit XmpText    10  normalized
""",
        "",
    ]

    stderr = [
        "",
        """$filename: No Exif data found in the file
$filename: No IPTC data found in the file
""",
        "",
    ]
    retval = [0] * 3



# -*- coding: utf-8 -*-

from system_tests import CaseMeta, path


class ThrowOnInput(metaclass=CaseMeta):

    url = "http://dev.exiv2.org/issues/841"

    filename = path("$data_path/exiv2-bug841.png")

    commands = ["$exiv2 $filename"]

    stdout = [""]
    stderr = [
        """$exiv2_exception_message $filename:
$kerFailedToReadImageData
"""
    ]
    retval = [1]



# -*- coding: utf-8 -*-

from system_tests import CaseMeta, CopyFiles, path


@CopyFiles("$data_path/exiv2-empty.jpg")
class FixMetadataDifferenceToExiftool(metaclass=CaseMeta):

    url = "http://dev.exiv2.org/issues/935"

    filename = path("$data_path/exiv2-empty_copy.jpg")

    commands = [
        """$exiv2 -u -v -M"set Exif.Photo.ExposureTime 605/10" $filename""",
        """$exiv2 -u -pv -gExif.Photo.ExposureTime $filename""",
        """$exiv2 -u -pa -gExif.Photo.ExposureTime $filename""",
        """$exiv2 -u -v -M"set Exif.Photo.ExposureTime 2/3" $filename""",
        """$exiv2 -u -pv -gExif.Photo.ExposureTime $filename""",
        """$exiv2 -u -pa -gExif.Photo.ExposureTime $filename""",
        """$exiv2 -u -v -M"set Exif.Photo.ExposureTime 0/0" $filename""",
        """$exiv2 -u -pv -gExif.Photo.ExposureTime $filename""",
        """$exiv2 -u -pa -gExif.Photo.ExposureTime $filename""",
        """$exiv2 -u -v -M"set Exif.Photo.ExposureTime 605/605" $filename""",
        """$exiv2 -u -pv -gExif.Photo.ExposureTime $filename""",
        """$exiv2 -u -pa -gExif.Photo.ExposureTime $filename""",
        """$exiv2 -u -v -M"set Exif.Photo.ExposureTime 100/1500" $filename""",
        """$exiv2 -u -pv -gExif.Photo.ExposureTime $filename""",
        """$exiv2 -u -pa -gExif.Photo.ExposureTime $filename""",
        """$exiv2 -u -v -M"set Exif.Photo.ExposureTime Ascii Test" $filename""",
        """$exiv2 -u -pv -gExif.Photo.ExposureTime $filename""",
        """$exiv2 -u -pa -gExif.Photo.ExposureTime $filename""",
    ]

    stdout = [
        """File 1/1: $filename
Set Exif.Photo.ExposureTime "605/10" (Rational)
""",
        """0x829a Photo        ExposureTime                Rational    1  605/10
""",
        """Exif.Photo.ExposureTime                      Rational    1  60.5 s
""",
        """File 1/1: $filename
Set Exif.Photo.ExposureTime "2/3" (Rational)
""",
        """0x829a Photo        ExposureTime                Rational    1  2/3
""",
        """Exif.Photo.ExposureTime                      Rational    1  0.666667 s
""",
        """File 1/1: $filename
Set Exif.Photo.ExposureTime "0/0" (Rational)
""",
        """0x829a Photo        ExposureTime                Rational    1  0/0
""",
        """Exif.Photo.ExposureTime                      Rational    1  (0/0)
""",
        """File 1/1: $filename
Set Exif.Photo.ExposureTime "605/605" (Rational)
""",
        """0x829a Photo        ExposureTime                Rational    1  605/605
""",
        """Exif.Photo.ExposureTime                      Rational    1  1 s
""",
        """File 1/1: $filename
Set Exif.Photo.ExposureTime "100/1500" (Rational)
""",
        """0x829a Photo        ExposureTime                Rational    1  100/1500
""",
        """Exif.Photo.ExposureTime                      Rational    1  1/15 s
""",
        """File 1/1: $filename
Set Exif.Photo.ExposureTime "Test" (Ascii)
""",
        """0x829a Photo        ExposureTime                Ascii       5  Test
""",
        """Exif.Photo.ExposureTime                      Ascii       5  (Test)
"""
    ]

    stderr = [""] * 18

    retval = [0] * 18



# -*- coding: utf-8 -*-

from system_tests import CaseMeta, CopyTmpFiles, path, check_no_ASAN_UBSAN_errors
@CopyTmpFiles("$data_path/issue_ghsa_mxw9_qx4c_6m8v_poc.jp2")

class Jp2ImageEncodeJp2HeaderOutOfBoundsRead2(metaclass=CaseMeta):
    """
    Regression test for the bug described in:
    https://github.com/Exiv2/exiv2/security/advisories/GHSA-mxw9-qx4c-6m8v
    """
    url = "https://github.com/Exiv2/exiv2/security/advisories/GHSA-mxw9-qx4c-6m8v"

    filename = path("$tmp_path/issue_ghsa_mxw9_qx4c_6m8v_poc.jp2")
    commands = ["$exiv2 rm $filename"]
    stdout = [""]
    retval = [0]

    compare_stderr = check_no_ASAN_UBSAN_errors



# -*- coding: utf-8 -*-

from system_tests import CaseMeta, path


class OutOfBoundsReadInIptcParserDecode(metaclass=CaseMeta):
    """
    Regression test for the bug described in:
    https://github.com/Exiv2/exiv2/issues/869
    """
    url = "https://github.com/Exiv2/exiv2/issues/869"

    filename = path("$data_path/issue_869_poc.png")
    commands = ["$exiv2 $filename"]
    stdout = [""]
    stderr = [
        """Exiv2 exception in print action for file $filename:
Failed to read image data
"""
]
    retval = [1]



# -*- coding: utf-8 -*-

import system_tests
import unittest

@unittest.skip("Skipping test using option -pR (only for Debug mode)")
class UncontrolledRecursion(metaclass=system_tests.CaseMeta):
    url = "https://github.com/Exiv2/exiv2/issues/216"

    filename = system_tests.path(
        "$data_path/"
        "exiv2_0-26_exiv2_uncontrolled-recursion_printIFDStructure.tif"
    )

    commands = ["$exiv2 -pR $filename"]
    retval = [1]
    stdout = [
        """STRUCTURE OF TIFF FILE (MM): $filename
 address |    tag                              |      type |    count |    offset | value
      10 | 0x0100 ImageWidth                   |     SHORT |        1 |           | 1
      22 | 0x0103 Compression                  |     SHORT |        0 |           | 
      34 | 000000 GPSVersionID                 |      BYTE |        0 |           | """
    ]

    compare_stderr = system_tests.check_no_ASAN_UBSAN_errors



# -*- coding: utf-8 -*-

import itertools

import system_tests


TYPES = ["8BIM", "AgHg", "DCSR", "PHUT"]
FORMATS = ["jpg", "psd"]


def make_commands(filename):
    return [
        """$exiv2 -u -v -M"set Exif.Photo.UserComment Test" """ + filename,
        "$exiv2 -u -pt -b " + filename
    ]


def make_filename(irb_type, img_format):
    return "$data_path/exiv2-bug800-" + irb_type + "_copy." + img_format


def make_stdout(irb_type, img_format):
    first = """File 1/1: """ + make_filename(irb_type, img_format) + """
Set Exif.Photo.UserComment "Test" (Comment)
"""

    if img_format == "jpg":
        second = """Exif.Image.Orientation                       Short       1  top, left
Exif.Image.XResolution                       Rational    1  180
Exif.Image.YResolution                       Rational    1  180
Exif.Image.ResolutionUnit                    Short       1  inch
Exif.Image.Software                          Ascii      16  QuickTime 6.5.2
Exif.Image.DateTime                          Ascii      20  2005:03:27 19:46:52
Exif.Image.HostComputer                      Ascii      16  Mac OS X 10.3.8
Exif.Image.YCbCrPositioning                  Short       1  Centered
Exif.Image.ExifTag                           Long        1  190
Exif.Photo.ExposureTime                      Rational    1  1/200 s
Exif.Photo.FNumber                           Rational    1  F7.1
Exif.Photo.ExifVersion                       Undefined   4  2.20
Exif.Photo.DateTimeOriginal                  Ascii      20  2005:03:18 22:15:08
Exif.Photo.DateTimeDigitized                 Ascii      20  2005:03:18 22:15:08
Exif.Photo.ComponentsConfiguration           Undefined   4  YCbCr
Exif.Photo.CompressedBitsPerPixel            Rational    1  5
Exif.Photo.ShutterSpeedValue                 SRational   1  1/202 s
Exif.Photo.ApertureValue                     Rational    1  F7.1
Exif.Photo.ExposureBiasValue                 SRational   1  0 EV
Exif.Photo.MaxApertureValue                  Rational    1  F2.8
Exif.Photo.MeteringMode                      Short       1  Multi-segment
Exif.Photo.Flash                             Short       1  No, auto
Exif.Photo.FocalLength                       Rational    1  7.4 mm
Exif.Photo.MakerNote                         Undefined 590  14 0 1 0 3 0 46 0 0 0 92 4 0 0 2 0 3 0 4 0 0 0 184 4 0 0 3 0 3 0 4 0 0 0 192 4 0 0 4 0 3 0 34 0 0 0 200 4 0 0 0 0 3 0 6 0 0 0 12 5 0 0 0 0 3 0 4 0 0 0 24 5 0 0 18 0 3 0 28 0 0 0 32 5 0 0 19 0 3 0 4 0 0 0 88 5 0 0 6 0 2 0 32 0 0 0 96 5 0 0 7 0 2 0 24 0 0 0 128 5 0 0 8 0 4 0 1 0 0 0 104 167 20 0 9 0 2 0 32 0 0 0 152 5 0 0 16 0 4 0 1 0 0 0 0 0 39 1 13 0 3 0 34 0 0 0 184 5 0 0 0 0 0 0 92 0 2 0 0 0 5 0 1 0 0 0 0 0 4 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 15 0 3 0 1 0 1 64 0 0 255 255 255 255 199 2 237 0 32 0 98 0 180 0 0 0 0 0 0 0 0 0 0 0 0 0 255 255 0 0 224 8 224 8 0 0 1 0 0 0 0 0 255 127 0 0 0 0 0 0 2 0 237 0 30 1 215 0 0 4 0 0 0 0 0 0 68 0 0 0 128 0 50 1 181 0 245 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 121 0 0 0 180 0 247 0 0 0 0 0 0 0 250 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 9 0 9 0 224 8 168 6 224 8 212 0 153 1 38 0 102 254 0 0 154 1 102 254 0 0 154 1 102 254 0 0 154 1 215 255 215 255 215 255 0 0 0 0 0 0 41 0 41 0 41 0 16 0 4 0 0 0 0 0 0 0 0 0 73 77 71 58 80 111 119 101 114 83 104 111 116 32 83 52 48 48 32 74 80 69 71 0 0 0 0 0 0 0 0 0 70 105 114 109 119 97 114 101 32 86 101 114 115 105 111 110 32 50 46 48 48 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 68 0 9 0 67 1 57 1 62 1 65 1 69 1 68 1 67 1 66 1 70 1 64 0 0 0 0 0 83 0 0 0 0 0 10 0 0 0 10 0 3 0 120 0 123 1 26 0 249 255 249 3 10 4 0 0 0 0 0 0 0 0 153 0 0 0 0 0
Exif.Photo.UserComment                       Undefined  12  Test
Exif.Photo.FlashpixVersion                   Undefined   4  1.00
Exif.Photo.PixelXDimension                   Short       1  2272
Exif.Photo.PixelYDimension                   Short       1  1704
Exif.Photo.FocalPlaneXResolution             Rational    1  8114.29
Exif.Photo.FocalPlaneYResolution             Rational    1  8114.29
Exif.Photo.FocalPlaneResolutionUnit          Short       1  inch
Exif.Photo.SensingMethod                     Short       1  One-chip color area
Exif.Photo.FileSource                        Undefined   1  Digital still camera
Exif.Photo.CustomRendered                    Short       1  Normal process
Exif.Photo.ExposureMode                      Short       1  Auto
Exif.Photo.WhiteBalance                      Short       1  Auto
Exif.Photo.DigitalZoomRatio                  Rational    1  1.0
Exif.Photo.SceneCaptureType                  Short       1  Standard
"""
    elif img_format == 'psd':
        second = """Exif.Image.ImageWidth                        Short       1  150
Exif.Image.ImageLength                       Short       1  91
Exif.Image.BitsPerSample                     Short       3  8 8 8
Exif.Image.Orientation                       Short       1  top, left
Exif.Image.SamplesPerPixel                   Short       1  3
Exif.Image.XResolution                       Rational    1  72
Exif.Image.YResolution                       Rational    1  72
Exif.Image.ResolutionUnit                    Short       1  inch
Exif.Image.Software                          Ascii      30  Adobe Photoshop CS5 Macintosh
Exif.Image.DateTime                          Ascii      20  2011:06:27 21:41:02
Exif.Image.ExifTag                           Long        1  218
Exif.Photo.ExifVersion                       Undefined   4  2.21
Exif.Photo.UserComment                       Undefined  12  Test
Exif.Photo.ColorSpace                        Short       1  Uncalibrated
Exif.Photo.PixelXDimension                   Long        1  150
Exif.Photo.PixelYDimension                   Long        1  91
Exif.Thumbnail.Compression                   Short       1  JPEG (old-style)
Exif.Thumbnail.XResolution                   Rational    1  72
Exif.Thumbnail.YResolution                   Rational    1  72
Exif.Thumbnail.ResolutionUnit                Short       1  inch
Exif.Thumbnail.JPEGInterchangeFormat         Long        1  390
Exif.Thumbnail.JPEGInterchangeFormatLength   Long        1  0
"""
    return [first, second]


FILES = [
    make_filename(irb_type, img_format)
    for irb_type, img_format in itertools.product(TYPES, FORMATS)
]

ORIGINAL_FILES = [
    "$data_path/exiv2-bug800-" + irb_type + "." + img_format
    for irb_type, img_format in itertools.product(TYPES, FORMATS)
]


@system_tests.CopyFiles(*ORIGINAL_FILES)
class MissingPhotoshopIrbTypes(metaclass=system_tests.CaseMeta):

    url = "http://dev.exiv2.org/issues/800"

    types = ["8BIM", "AgHg", "DCSR", "PHUT"]
    formats = ["jpg", "psd"]

    commands = list(
        itertools.chain.from_iterable(
            make_commands(fname) for fname in FILES
        )
    )

    stdout = list(
        itertools.chain.from_iterable(
            make_stdout(irb_type, img_format)
            for irb_type, img_format in itertools.product(TYPES, FORMATS)
        )
    )
    stderr = [""] * 16
    retval = [0] * 16



# -*- coding: utf-8 -*-

import os
import shutil
import string

import system_tests


class FailureOnCifsShares(metaclass=system_tests.CaseMeta):

    url = "http://dev.exiv2.org/issues/1043"
    num = 1043

    original_file = system_tests.path("$data_path/exiv2-bug884c.jpg")

    files = [
        system_tests.path("$data_path/bug$num-" + char + ".jpg")
        for char in string.ascii_uppercase
    ]

    def setUp(self):
        for fname in self.files:
            shutil.copyfile(self.original_file, fname)

    def tearDown(self):
        for fname in self.files:
            os.remove(fname)

    commands = [
        """$exiv2 -u -v -M"set Exif.Photo.UserComment Test Bug $num my filename is {fname_short}" {fname}"""\
        .format(fname=fname, fname_short=os.path.split(fname)[1])
        for fname in files
    ] + [
        # workaround for * wildcard in bash:
        """$exiv2 -PE -g UserComment {!s}""".format(" ".join(files))
    ]

    retval = [0] * (len(files) + 1)
    stdout = [
        """File 1/1: {fname}
Set Exif.Photo.UserComment "Test Bug $num my filename is {fname_short}" (Comment)
"""
        .format(fname=fname, fname_short=os.path.split(fname)[1])
        for fname in files
    ] + [
        """""".join(
            """{fname}  Exif.Photo.UserComment                       Undefined  50  Test Bug $num my filename is {fname_short}
"""
            .format(fname=fname, fname_short=os.path.split(fname)[1])
            for fname in files
        )
    ]

    stderr= [""] * (len(files) + 1)



# -*- coding: utf-8 -*-

from system_tests import CaseMeta, path, check_no_ASAN_UBSAN_errors


class InvalidDataLocationInCiffComponentDoRead(metaclass=CaseMeta):
    """
    Regression test for the bug described in:
    https://github.com/Exiv2/exiv2/issues/841

    An invalid data location causes an assertion failure.
    """
    url = "https://github.com/Exiv2/exiv2/issues/841"

    filename = path("$data_path/issue_841_poc.crw")
    commands = ["$exiv2 $filename"]
    stdout = [""]
    stderr = [
        """$exiv2_exception_message $filename:
$kerCorruptedMetadata
"""]
    retval = [1]



# -*- coding: utf-8 -*-

from system_tests import CaseMeta, path


class Jp2ImageDoWriteMetadataOutOfBoundsRead(metaclass=CaseMeta):
    """
    Regression test for the bug described in:
    https://github.com/Exiv2/exiv2/security/advisories/GHSA-7569-phvm-vwc2
    """
    url = "https://github.com/Exiv2/exiv2/security/advisories/GHSA-7569-phvm-vwc2"

    filename1 = path("$data_path/issue_ghsa_7569_phvm_vwc2_poc.jp2")
    filename2 = path("$data_path/issue_ghsa_7569_phvm_vwc2_poc.exv")
    commands = ["$exiv2 -q in $filename1"]
    stdout = [""]
    stderr = [
"""Exiv2 exception in insert action for file $filename1:
$kerCorruptedMetadata
"""]
    retval = [1]



# -*- coding: utf-8 -*-

import system_tests

class ShouldNotThrowsWithSpecificIsoSpeedValue(metaclass=system_tests.CaseMeta):
    url = "https://github.com/Exiv2/exiv2/issues/561"

    filename = system_tests.path("$data_path/pocIssue561")
    commands = ["$exiv2 " + filename]
    retval = [0]
    compare_stderr = system_tests.check_no_ASAN_UBSAN_errors
    compare_stdout = system_tests.check_no_ASAN_UBSAN_errors



# -*- coding: utf-8 -*-

import os
import shutil

import system_tests


class DoNotDestroyHardLinks(metaclass=system_tests.CaseMeta):

    def setUp(self):
        shutil.copy(self.orig_file, self.filename)
        os.link(self.filename, self.link_1)
        os.link(self.filename, self.link_2)

    def tearDown(self):
        for f in [self.filename, self.link_1, self.link_2]:
            os.remove(f)

    url = "http://dev.exiv2.org/issues/812"
    num = 812

    # original file, copy & hardlinks
    orig_file = "$data_path/exiv2-bug884c.jpg"
    filename = "$data_path/exiv2-bug$num.jpg"
    link_1 = "$data_path/exiv2-bug$num-B.jpg"
    link_2 = "$data_path/exiv2-bug$num-C.jpg"

    # list of files passed to exiv2 since we can't use * in the command
    file_list = "{!s} {!s} {!s}".format(filename, link_1,  link_2)

    commands = [
        """$exiv2 -u -v -M"set Exif.Photo.UserComment Test Bug $num" $filename""",
        "$exiv2 -PE -g UserComment $file_list",
        """$exiv2 -u -v -M"set Exif.Photo.UserComment Test Bug $num modified" $filename""",
        "$exiv2 -PE -g UserComment $file_list"
    ]

    first_change = "Exif.Photo.UserComment                       Undefined  20  Test Bug 812"
    second_change = "Exif.Photo.UserComment                       Undefined  29  Test Bug 812 modified"

    stdout = [
        """File 1/1: $filename
Set Exif.Photo.UserComment "Test Bug 812" (Comment)
""",
        """$filename  $first_change
$link_1  $first_change
$link_2  $first_change
""",
        """File 1/1: $filename
Set Exif.Photo.UserComment "Test Bug 812 modified" (Comment)
""",
        """$filename  $second_change
$link_1  $second_change
$link_2  $second_change
"""
    ]
    stderr = [""] * 4
    retval = [0] * 4



# -*- coding: utf-8 -*-

from system_tests import CaseMeta, path

class Tamron_17_35_Di_OSD(metaclass=CaseMeta):

    filename = path("$data_path/exiv2-pr1105.exv")
    commands = ["$exiv2 -pa --grep lens/i $filename"]

    stdout = ["""Exif.Nikon3.LensType                         Byte        1  D G 
Exif.Nikon3.Lens                             Rational    4  17-35mm F2.8-4
Exif.Nikon3.LensFStops                       Undefined   4  5
Exif.NikonLd3.LensIDNumber                   Byte        1  Tamron 17-35mm F/2.8-4 Di OSD
Exif.NikonLd3.LensFStops                     Byte        1  F5.0
"""
    ]
    stderr = [""]
    retval = [0]



# -*- coding: utf-8 -*-

import system_tests

class CheckTokina11_20mm(metaclass=system_tests.CaseMeta):

    url = "http://dev.exiv2.org/issues/1166"

    filename = system_tests.path("$data_path/exiv2-bug1166.exv")

    commands = [ "$exiv2 -pa --grep lens/i $filename" ]

    stdout = [ """Exif.CanonCs.LensType                        Short       1  Tokina AT-X 11-20 F2.8 PRO DX Aspherical 11-20mm f/2.8
Exif.CanonCs.Lens                            Short       3  11.0 - 20.0 mm
Exif.Canon.LensModel                         Ascii      74  11-20mm
Exif.Photo.LensSpecification                 Rational    4  11/1 20/1 0/1 0/1
Exif.Photo.LensModel                         Ascii       8  11-20mm
Exif.Photo.LensSerialNumber                  Ascii      11  0000000000
""",
    ]
    stderr = [""]
    retval = [0]



# -*- coding: utf-8 -*-

from system_tests import CaseMeta, FileDecoratorBase, path
from struct import *

# The PoC is a fairly large file, mostly consisting of zero bytes,
# so it would be a waste of storage to check it into the repo.
# Instead, we can generate the PoC with a small amount of code:
class CreatePoC(FileDecoratorBase):
    """
    This class copies files from test/data to test/tmp
    Copied files are NOT removed in tearDown
    Example: @CopyTmpFiles("$data_path/test_issue_1180.exv")
    """

    #: override the name of the file list
    FILE_LIST_NAME = '_tmp_files'

    def setUp_file_action(self, expanded_file_name):
        size = 0x20040
        contents = pack('<2sI8sHHIIHHII', bytes(b'II'), 14, bytes(b'HEAPCCDR'), \
                        1, 0x300b, size - 26, 12, 1, 0x102a, size - 38, 12) + \
                        bytes(bytearray(size-38))
        f = open(expanded_file_name, 'wb')
        f.write(contents)
        f.close()

    def tearDown_file_action(self, f):
        """
        Do nothing.   We don't clean up TmpFiles
        """

# This decorator generates the PoC file.
@CreatePoC("$tmp_path/issue_ghsa_hqjh_hpv8_8r9p_poc.crw")

class CrwMapDecodeArrayInfiniteLoop(metaclass=CaseMeta):
    """
    Regression test for the bug described in:
    https://github.com/Exiv2/exiv2/security/advisories/GHSA-hqjh-hpv8-8r9p
    """
    url = "https://github.com/Exiv2/exiv2/security/advisories/GHSA-hqjh-hpv8-8r9p"

    filename = path("$tmp_path/issue_ghsa_hqjh_hpv8_8r9p_poc.crw")

    commands = ["$exiv2 $filename"]
    stdout = [""]
    stderr = [
"""Exiv2 exception in print action for file $filename:
$kerCorruptedMetadata
"""]
    retval = [1]



# -*- coding: utf-8 -*-

from system_tests import CaseMeta, path


class NewTamronAndPentaxLenses(metaclass=CaseMeta):

    url = "http://dev.exiv2.org/issues/884"

    filenames = [
        path("$data_path/exiv2-bug884{:s}.jpg".format(char))
        for char in ["a", "b", "c"]
    ]

    commands = [
        "$exiv2 -pt --grep LensType " + fname for fname in filenames
    ]

    stdout = [
        """Exif.Pentax.LensType                         Byte        2  Tamron SP AF 17-50mm F2.8 XR Di II
""",
        """Exif.Pentax.LensType                         Byte        2  smc PENTAX-F 35-70mm F3.5-4.5
""",
        """Exif.Pentax.LensType                         Byte        2  PENTAX-F 28-80mm F3.5-4.5
"""
    ]
    stderr = [""] * len(commands)
    retval = [0] * len(commands)



# -*- coding: utf-8 -*-

import system_tests


class CrashAfterLightzoneEdit(metaclass=system_tests.CaseMeta):

    url = "http://dev.exiv2.org/issues/479"

    filename = "$data_path/exiv2-bug479.jpg"

    commands = ["$exiv2 -u -b -pt $filename"]

    stdout = ["""Exif.Image.ImageWidth                        Short       1  3173
Exif.Image.ImageLength                       Short       1  2011
Exif.Image.Software                          Ascii      10  LightZone
Exif.Image.HostComputer                      Ascii      20  Linux 2.6.15-23-686
Exif.Image.ExposureTime                      Rational    1  3.2 s
Exif.Image.FNumber                           Rational    1  F9
Exif.Image.ExifTag                           Long        1  784
Exif.Photo.ExifVersion                       Undefined   4  2.20
Exif.Photo.PixelXDimension                   Short       1  3173
Exif.Photo.PixelYDimension                   Short       1  2011
Exif.Image.ExposureProgram                   Short       1  Manual
Exif.Image.ISOSpeedRatings                   Short       1  250
Exif.Image.0x9000                            Undefined   4  48 50 50 49
Exif.Image.DateTimeOriginal                  Ascii      20  2006:05:27 12:37:03
Exif.Image.0x9004                            Ascii      20  2006:05:27 12:37:03
Exif.Image.0x9101                            Undefined   4  0 0 0 0
Exif.Image.ShutterSpeedValue                 SRational   1  3 s
Exif.Image.ApertureValue                     Rational    1  F9
Exif.Image.ExposureBiasValue                 SRational   1  0 EV
Exif.Image.MeteringMode                      Short       1  Multi-segment
Exif.Image.Flash                             Short       1  No, compulsory
Exif.Image.FocalLength                       Rational    1  24.0 mm
Exif.Image.0x9286                            Undefined 264  0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0
"""]
    stderr = ["""Error: Directory Image: Next pointer is out of bounds; ignored.
"""]
    retval = [0]



# -*- coding: utf-8 -*-

import system_tests

class Canon_Sigma_18_35_F18_DC_HSM(metaclass=system_tests.CaseMeta):
    url = "https://github.com/Exiv2/exiv2/issues/1368"
    
    filename1 = "$data_path/Canon_Sigma_18_35_F18_DC_HSM_firmware_1xx.exv"
    filename2 = "$data_path/Canon_Sigma_18_35_F18_DC_HSM_firmware_2xx.exv"
    commands  = ["$exiv2 -pa -K Exif.CanonCs.LensType $filename1",
                 "$exiv2 -pa -K Exif.CanonCs.LensType $filename2"]
    stderr = ["", ""]
    stdout = [
        """Exif.CanonCs.LensType                        Short       1  Sigma 18-35mm f/1.8 DC HSM | A
""",
        """Exif.CanonCs.LensType                        Short       1  Sigma 18-35mm f/1.8 DC HSM | A
"""
]
    retval = [0, 0]



# -*- coding: utf-8 -*-

import system_tests

class CheckSigma17_70Lens(metaclass=system_tests.CaseMeta):

    url = "http://dev.exiv2.org/issues/1167"

    filename = system_tests.path("$data_path/exiv2-bug1167.exv")

    commands = [ "$exiv2 -pa --grep lens/i $filename" ]

    stdout = [ """Exif.CanonCs.LensType                        Short       1  Sigma 17-70mm f/2.8-4 DC Macro OS HSM
Exif.CanonCs.Lens                            Short       3  17.0 - 70.0 mm
Exif.Canon.LensModel                         Ascii      74  17-70mm
Exif.Photo.LensSpecification                 Rational    4  17/1 70/1 0/1 0/1
Exif.Photo.LensModel                         Ascii       8  17-70mm
Exif.Photo.LensSerialNumber                  Ascii      11  0000000000
""",
    ]
    stderr = [""]
    retval = [0]



# -*- coding: utf-8 -*-

import system_tests


@system_tests.CopyFiles("$data_path/exiv2-bug528.jpg")
class TypeSizeForExifOnly(metaclass=system_tests.CaseMeta):

    url = "http://dev.exiv2.org/issues/528"

    filename = "$data_path/exiv2-bug528_copy.jpg"

    commands = [
        "$exiv2 -u -b -pt $filename",
        """$exiv2 -u -v -M"set Exif.Image.Software GI" $filename"""
    ]

    stdout = ["""Exif.Image.ProcessingSoftware                Ascii      20  digiKam-0.9.4-beta4
Exif.Image.ImageWidth                        Long        1  2888
Exif.Image.ImageLength                       Long        1  1700
Exif.Image.DocumentName                      Ascii      13  Chambord.jpg
Exif.Image.Make                              Ascii       6  Canon
Exif.Image.Model                             Ascii      14  Canon EOS 40D
Exif.Image.Orientation                       Short       1  top, left
Exif.Image.XResolution                       Rational    1  72
Exif.Image.YResolution                       Rational    1  72
Exif.Image.ResolutionUnit                    Short       1  inch
Exif.Image.Software                          Ascii      11  GIMP 2.4.5
Exif.Image.DateTime                          Ascii      20  2008:04:14 08:24:04
Exif.Image.YCbCrPositioning                  Short       1  Centered
Exif.Image.Rating                            SLong       1  3
Exif.Image.RatingPercent                     SLong       1  50
Exif.Image.ExifTag                           Long        1  320
Exif.Photo.ExposureTime                      Rational    1  1/250 s
Exif.Photo.FNumber                           Rational    1  F11
Exif.Photo.ExposureProgram                   Short       1  Auto
Exif.Photo.ISOSpeedRatings                   Short       1  400
Exif.Photo.ExifVersion                       Undefined   4  2.21
Exif.Photo.DateTimeOriginal                  Ascii      20  2008:04:01 13:40:03
Exif.Photo.DateTimeDigitized                 Ascii      20  2008:04:01 13:40:03
Exif.Photo.ComponentsConfiguration           Undefined   4  YCbCr
Exif.Photo.ShutterSpeedValue                 SRational   1  1/256 s
Exif.Photo.ApertureValue                     Rational    1  F11
Exif.Photo.ExposureBiasValue                 SRational   1  +1/3 EV
Exif.Photo.MeteringMode                      Short       1  Multi-segment
Exif.Photo.Flash                             Short       1  No, compulsory
Exif.Photo.FocalLength                       Rational    1  10.0 mm
Exif.Photo.MakerNote                         Undefined  18  0 1 2 0 3 0 0 0 0 0 0 0 0 0 0 0 0 0
Exif.MakerNote.Offset                        Long        1  798
Exif.MakerNote.ByteOrder                     Ascii       3  MM
Exif.Canon.0x0200                            0x0300      0  
Exif.Photo.UserComment                       Undefined  38  Chateaux de la Loire, Chambord
Exif.Photo.SubSecTime                        Ascii       3  81
Exif.Photo.SubSecTimeOriginal                Ascii       3  81
Exif.Photo.SubSecTimeDigitized               Ascii       3  81
Exif.Photo.FlashpixVersion                   Undefined   4  1.00
Exif.Photo.ColorSpace                        Short       1  sRGB
Exif.Photo.PixelXDimension                   Long        1  2888
Exif.Photo.PixelYDimension                   Long        1  1700
Exif.Photo.FocalPlaneXResolution             Rational    1  4438.36
Exif.Photo.FocalPlaneYResolution             Rational    1  4445.97
Exif.Photo.FocalPlaneResolutionUnit          Short       1  inch
Exif.Photo.CustomRendered                    Short       1  Normal process
Exif.Photo.ExposureMode                      Short       1  Auto
Exif.Photo.WhiteBalance                      Short       1  Auto
Exif.Photo.SceneCaptureType                  Short       1  Standard
Exif.Photo.Contrast                          Short       1  Normal
Exif.Photo.Saturation                        Short       1  Normal
Exif.Image.GPSTag                            Long        1  870
Exif.GPSInfo.GPSVersionID                    Byte        4  2.0.0.0
Exif.GPSInfo.GPSLatitudeRef                  Ascii       2  North
Exif.GPSInfo.GPSLatitude                     Rational    3  47deg 36' 58"
Exif.GPSInfo.GPSLongitudeRef                 Ascii       2  East
Exif.GPSInfo.GPSLongitude                    Rational    3  1deg 31' 1"
Exif.GPSInfo.GPSAltitudeRef                  Byte        1  Above sea level
Exif.GPSInfo.GPSAltitude                     Rational    1  86 m
Exif.Thumbnail.Compression                   Short       1  JPEG (old-style)
Exif.Thumbnail.JPEGInterchangeFormat         Long        1  1058
Exif.Thumbnail.JPEGInterchangeFormatLength   Long        1  4492
""",
        """File 1/1: $filename
Set Exif.Image.Software "GI" (Ascii)
"""
    ]
    stderr = [
        """Warning: Directory Canon, entry 0x0200 has unknown Exif (TIFF) type 768; setting type size 1.
""",
        """Warning: Directory Canon, entry 0x0200 has unknown Exif (TIFF) type 768; setting type size 1.
Warning: Directory Canon, entry 0x0200 has unknown Exif (TIFF) type 768; setting type size 1.
"""
    ]
    retval = [0] * 2

    compare_stderr = system_tests.check_no_ASAN_UBSAN_errors



# -*- coding: utf-8 -*-

import system_tests

@system_tests.CopyFiles("$data_path/exiv2-empty.jpg")
class CheckXmlLang(metaclass=system_tests.CaseMeta):

    url = "http://dev.exiv2.org/issues/1058"

    filename = system_tests.path("$data_path/exiv2-empty_copy.jpg")
    commands = [
        ## Add titles in 2 languages and one default
        """$exiv2 -M"set Xmp.dc.title lang=de-DE GERMAN"  $filename""",
        """$exiv2 -M"set Xmp.dc.title lang=en-GB BRITISH" $filename""",
        """$exiv2 -M"set Xmp.dc.title Everybody else"     $filename""",
        """$exiv2 -px                                     $filename""",
        ## Remove languages, test case for the language
        """$exiv2 -M"set Xmp.dc.title lang=DE-de german"  $filename""",
        """$exiv2 -M"set Xmp.dc.title lang=EN-gb"         $filename""",
        """$exiv2 -M"set Xmp.dc.title"                    $filename""",
        """$exiv2 -px                                     $filename""",
    ]

    stdout = [
        "",
        "",
        "",
        """Xmp.dc.title                                 LangAlt     3  lang="x-default" Everybody else, lang="en-GB" BRITISH, lang="de-DE" GERMAN
""",
        "",
        "",
        "",
        """Xmp.dc.title                                 LangAlt     1  lang="de-DE" german
""",
             ]

    stderr = [""] * len(commands)
    retval = [0] * len(commands)



# -*- coding: utf-8 -*-

import system_tests


class TestCvePoC(metaclass=system_tests.CaseMeta):

    url = "https://github.com/Exiv2/exiv2/issues/144"

    filename = "$data_path/POC1"
    commands = ["$exiv2 $filename"]
    stdout = [
        """File name       : $filename
File size       : 26 Bytes
MIME type       : image/tiff
Image size      : 0 x 0
"""
    ]
    stderr = [""]
    retval = ["$no_exif_data_found_retval"]

    compare_stderr = system_tests.check_no_ASAN_UBSAN_errors
